<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DTO\Response\Message;

class PDV extends CrudWithParameterBag {
    
    protected $PDVService;
    protected $util;
    protected $dataSourceNameArray = array (
        'pdv'                  => 'pdv',
        'modPDV'               => 'modPDV',
        'tef'                  => 'vnd42300_cadcaixa_tef',
        'aberCaixa'            => 'vnd42300_cadcaixa_abercaixa',
        'fechCaixa'            => 'vnd42300_cadcaixa_fechcaixa',
        'frentCaixa1'          => 'vnd42300_cadcaixa_frentedecaixa',
        'frentCaixa2'          => 'vnd42300_cadcaixa_frentedecaixa2',
        'frentCaixa3'          => 'vnd42300_cadcaixa_frentedecaixa3',
        'caixaAssoc1'          => 'vnd42300_cadcaixa_caixasassociados',
        'caixaAssoc2'          => 'vnd42300_cadcaixa_caixasassociados2',
        'perifericos1'         => 'vnd42300_cadcaixa_perifericos',
        'perifericos2'         => 'vnd42300_cadcaixa_perifericos2',
        'impressora'           => 'get_params_impressora',
        'vendedoresAssociados' => 'vnd42300_cadcaixa_vendedoresAssociados',
        'vendedores'           => 'vnd42300_cadcaixa_vendedores',
        'setorSemFilial'       => 'select_setor_sem_filial'
    );
    protected $entityManager;
    protected $parameterBagColumnsArray = array (
        'caixa'                =>  array('CDFILIAL', 'NRORG', 'CDCAIXA'),
        'loja'                 =>  array('CDFILIAL', 'NRORG', 'CDLOJA'),
        'PDV'                  =>  array('CDFILIAL', 'NRORG','CDLOJA','CDCAIXA'),
        'vendedoresAssociados' =>  array('NRORG', 'CDCAIXA', 'CDFILIAL'),
        'vendedores'           =>  array('NRORG', 'CDCAIXA', 'CDFILIAL'),
        'setorSemFilial'       =>  array('NRORG')
    );

    public function __construct ($dataSourceManager, $PDVService, $parameterBag, $entityManager, $util) {
        $this->PDVService = $PDVService;
        $this->entityManager = $entityManager;
        $this->util = $util;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    public function saveDefaultPDV ($request, $response, $dataSourceNameSave) {
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName($dataSourceNameSave);
        $persistedRows = $this->dataSourceManager->persist($dataSet);
        $response->addDataSet(new DataSource\DataSet($dataSourceNameSave, $persistedRows));
    } 
    
    public function findTEF (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['tef'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function saveTEF (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['tef']);
    }
    
    public function findAberCaixa (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['aberCaixa'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function saveAberCaixa (DTO\Request $request, DTO\Response $response) {
        $data = $request->getDataSet()->getRows();
        $data[0]['VRABERCAIX'] = str_replace(".","",$data[0]['VRABERCAIX']);
        $data[0]['VRABERCAIX'] = str_replace(",",".",$data[0]['VRABERCAIX']);
        $this->dataSourceManager->persist(new DataSet($this->dataSourceNameArray['aberCaixa'], array($data[0])));
    }
    
    public function findFechCaixa (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['fechCaixa'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function saveFechCaixa (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['fechCaixa']);
    }
    
    public function findFrentCaixa1 (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['frentCaixa1'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function saveFrentCaixa1 (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['frentCaixa1']);
    }
    
    public function findFrentCaixa2 (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['frentCaixa2'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function saveFrentCaixa2 (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['frentCaixa2']);
    }
    
    public function findFrentCaixa3 (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['frentCaixa3'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function saveFrentCaixa3 (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['frentCaixa3']);
    }
    
    public function findPerifericos1 (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['perifericos1'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function savePerifericos1 (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['perifericos1']);
    }
    
    public function findPerifericos2 (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['perifericos2'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['caixa'];
        parent::find($request, $response);
    }
    
    public function savePerifericos2 (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultPDV($request, $response, $this->dataSourceNameArray['perifericos2']);
    }
    
    public function findImpressora (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['impressora'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['loja'];
        parent::find($request, $response);
    }
    
    public function findPDV (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['pdv'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['PDV'];
        parent::find($request, $response);
    }
    
    public function savePDV (DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSet = $request->getDataSet();
        
        if (!array_key_exists(0, $dataSet->getRows()[0])) {
            $row = $dataSet->getRows()[count($request->getDataSet()->getRows())-1];
        } else {
            $row = $dataSet->getRows()[0][0]['pdv'];
        }
        
        $isNew = $row['__is_new'];
        
        if (!isset($row['CDATIVASAT'])) {
            $row['CDATIVASAT'] = null;
        }
        
        if (!isset($row['CDSAT'])) {
            $row['CDSAT'] = 0;
        }
        
        if (!isset($row['IDTPEQUSAT'])) {
            $row['IDTPEQUSAT'] = 'H';
        }
        
        if (!isset($row['CDSETOR'])) {
            $row['CDSETOR'] = null;
        }
        
        if (!isset($row['CDCAIXACONT'])) {
            $row['CDCAIXACONT'] = null;
        }
        
        $temCDCAIXA = $this->PDVService->findCDCaixa($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA'], $row['CDLOJA']);
        if (!$row['__is_new']) {
            $this->dataSourceManager->persist(new DataSet($this->dataSourceNameArray['pdv'], array($row)));
        } else if($this->PDVService->findCDCaixa($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA'], $row['CDLOJA'])) {
            $row['IDATIVO'] = 'S';  
            $this->dataSourceManager->persist(new DataSet($this->dataSourceNameArray['pdv'], array($row)));
        } else {
            try {
                if(($this->PDVService->findIDATIVO($row['CDFILIAL'], $row['CDLOJA'], $row['CDCAIXA'], $row['NRORG'])) == 'N') {
                    throw new \Exception("Este código de caixa já está registrado, porém inativo.");
                } 
                if($this->PDVService->findCD($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA'])){
                    throw new \Exception("Este código de caixa já está registrado nessa filial.");
                }
                else {
                    throw new \Exception("Este código de caixa já está registrado.");
                }
            } catch (\Exception $e) {
                $message = $e->getMessage();
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }  
        }
        
        if (array_key_exists('NRMODALIDADEPDV', $row) && $row['NRMODALIDADEPDV']) {
            $modArray = $row['NRMODALIDADEPDV'];
            $this->PDVService->deleteModalidades($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA']);
            $row['__is_new'] = true;
            foreach ($modArray as $modNr) {
                $row['NRMODALIDADEPDV'] = $modNr;
                $row['NRRELCAIXAMODALIDADE'] = $this->PDVService->getNextNumModalidade();
                $this->dataSourceManager->persist(new DataSet($this->dataSourceNameArray['modPDV'], array($row)));
            }
            $row['NRMODALIDADEPDV'] = $modArray;
        } else {
            $this->PDVService->deleteModalidades($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA']);
        }
    }

    public function deletePDV (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $row = $request->getRow();
            $this->PDVService->resetCaixa($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA'], $row['CDLOJA'], $row['DTULTATU'], $row['NRORGULTATU'], $row['CDOPERULTATU']);
            $this->entityManager->getConnection()->commit();   
        } catch (\Exception $e) {
            $message = $e->getMessage();
            $this->entityManager->getConnection()->rollBack(); 
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function exportarCaixas (DTO\Request $request, DTO\Response $response) {
        $row = $this->util->getParamsAsArray($request);

        $params = array(
            'NRORG'       => $row['NRORG'],
            'filiais'     => $row['CDFILIAL'],
            'caixas'      => $row['CDCAIXA'],
            'lojas'       => $row['CDLOJA'],
            'filiCaixas'  => $row['FILIALCAIXA'],
            'caixaAtual'  => $row['CAIXA_ATUAL'],
            'filialAtual' => $row['FILIAL_ATUAL'],
            'lojaAtual'   => $row['LOJA_ATUAL']
        );
        
        $this->PDVService->exportarCaixas($params);
    }
    
    public function exportarVendas (DTO\Request $request, DTO\Response $response) {
        $data = $request->getRow();
        $NRORG    = $data[0][2]['value'];
        $CDFILIAL = $data[0][0]['value'];
        $CDCAIXA  = $data[0][1]['value'];
        $this->PDVService->paramsImportExport($CDFILIAL, $CDCAIXA, $NRORG, $data, 'DSCAIXAVEN', Utilities\Query::UPDATE_PARAMS_EXPORT);
    }
    
    public function importarTabelas (DTO\Request $request, DTO\Response $response) {
        $data = $request->getRow();
        $NRORG    = $data[0][2]['value'];  
        $CDFILIAL = $data[0][0]['value'];
        $CDCAIXA  = $data[0][1]['value'];
        $CDCAIXA = str_replace("%", "", $CDCAIXA);
        $this->PDVService->paramsImportExport($CDFILIAL, $CDCAIXA, $NRORG, $data, 'DSCAIXATAB', Utilities\Query::UPDATE_PARAMS_IMPORT);
    }
    
    public function buscarCaixasImportacao (DTO\Request $request, DTO\Response $response) {
        
        $query = Utilities\Query::GET_PARAMS_EXPORT_IMPORT;
        $conditions = $this->util->getParamsAsArray($request);
        $conditions['CDCAIXA'] = str_replace("%", "", $conditions['CDCAIXA']);
       
        $row = $this->entityManager->getConnection()->fetchAll(Utilities\Query::CAIXA_IMPORTACAO, $conditions);
        $response->addDataSet(new DataSet($this->dataSourceNameArray['caixaAssoc2'], $row));
    }
    
    public function deletarTabelas (DTO\Request $request, DTO\Response $response) {
        $data = $request->getRow();
        $CDCAIXA = str_replace("%", "", $data[0][1]['value']);
        $params = array(
            ":CDFILIAL"  => $data[0][0]['value'],
	     	":CDCAIXA"   => $CDCAIXA,
	     	":NRORG"     => $data[0][2]['value']
        );
        $this->PDVService->deleteParamsImportExport($params, $data, 'DSCAIXATAB');
    }    
    
    public function deletarVendas (DTO\Request $request, DTO\Response $response) {
        $data = $request->getRow();
        $params = array(
            ":CDFILIAL"  => $data[0][0]['value'],
	     	":CDCAIXA"   => $data[0][1]['value'],
	     	":NRORG"     => $data[0][2]['value']
        );
        $this->PDVService->deleteParamsImportExport($params, $data, 'DSCAIXAVEN');
    }
    
    public function FindEI (DTO\Request $request, DTO\Response $response) {
        $query = Utilities\Query::GET_PARAMS_EXPORT_IMPORT;
        $conditions = $this->util->getParamsAsArray($request);
        $rowExport = $this->entityManager->getConnection()->fetchAll($query, $conditions);

        if(isset($rowExport[0]["DSCAIXAVEN"])) {
            $conditions["CDCAIXA"] = $rowExport[0]["DSCAIXAVEN"];
        } else {
            $conditions["CDCAIXA"] = "";
        }
        
        $row = $this->entityManager->getConnection()->fetchAll(Utilities\Query::CAIXA_BASICO, $conditions);
        $response->addDataSet(new DataSet($this->dataSourceNameArray['caixaAssoc1'], $row));
    }
    
    public function FindVT (DTO\Request $request, DTO\Response $response) {
        $query = Utilities\Query::GET_PARAMS_EXPORT_IMPORT;
        $conditions = $this->util->getParamsAsArray($request);
        $conditions['CDCAIXA'] = str_replace("%", "", $conditions['CDCAIXA']);
        $rowExport = $this->entityManager->getConnection()->fetchAll($query, $conditions);
       
      
        if(isset($rowExport[0]["DSCAIXATAB"])) {
            $conditions["CDCAIXA"] = $rowExport[0]["DSCAIXATAB"];
            
        } else {
            $conditions["CDCAIXA"] = "";
        }
       
        $row = $this->entityManager->getConnection()->fetchAll(Utilities\Query::CAIXA_BASICO, $conditions);
        $response->addDataSet(new DataSet($this->dataSourceNameArray['caixaAssoc2'], $row));
    }
    
    public function findVendedoresAssociados (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['vendedoresAssociados'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['vendedoresAssociados'];
        parent::find($request, $response);
    }
    
    public function findVendedores(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['vendedores'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['vendedores'];
        parent::find($request, $response);
    }

    public function verificaVendedores(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $codigos = array();
        $contaCodigos = 0;
        
        for($i = 0; $i < count($row['CDVENDEDOR']); $i++) {
            $codigos[] = $this->PDVService->verificaVendedor($row['NRORG'], $row['CDVENDEDOR'][$i]);
        }
        
        for($i = 0; $i < count($codigos); $i++) {
            if($codigos[$i] != NULL) {
                $contaCodigos++;
            }
        }
        
        if($contaCodigos > 0) {
            $temVendedoresRelacionados[] = true;
        } else {
            $temVendedoresRelacionados[] = false;
        }
        
        $response->addDataSet(new DataSet('temVendedoresRelacionados', $temVendedoresRelacionados));
    }
    
    public function relacionaVendedores(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        for($i = 0; $i < count($row['CDVENDEDOR']); $i++) {
            $this->PDVService->relacionaVendedor($row['NRORG'], $row['CDFILIAL'], $row['CDCAIXA'], $row['CDVENDEDOR'][$i]);
        }
    }
    
    public function desassociaVendedores(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $codigos = array();
        
        $NRORG = $row[0];
        $row = $row[1];
        
        for($i = 0; $i < count($row); $i++) {
            $codigos[] = $row[$i]['CDVENDEDOR'];
        }
        
        for($i = 0; $i < count($codigos); $i++) {
            $this->PDVService->desassociaVendedor($NRORG, $codigos[$i]);
        }
    }
    
    public function checkTipoDeCaixa(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $total = $this->PDVService->checkTipoDeCaixa($row);
        $response->addDataSet(new DataSet('totalRow', array('total'=>$total)));
    }
    
    public function findSetorSemFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['setorSemFilial'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['setorSemFilial'];
        parent::find($request, $response);
    }
}