<?php
namespace Teknisa\MAN\Factories;

use \Util\Exception;
use NFePHP\NFe\Tools;
use NFePHP\NFe\Make;
use NFePHP\NFe\Common\Standardize;
use NFePHP\Common\Certificate;
use NFePHP\Common\Strings;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\NFe\Complements;
use Teknisa\MAN\Controllers\TransmissaoNfceController;
use Teknisa\Libs\Util\Utilities;


class NotaFiscal {

    const AMB_EMISSAO_ONLINE = '1';
    const AMB_EMISSAO_OFFLINE = '9';
    const MODELO_DOC = '65'; // 55 - NFe; 65 - NFC-e
    const VERSAO_LAYOUT = '4.00';

    const APROVADA = 'O';
    const ENTRADA = 'E';
    const SAIDA = 'S';
    const PRODUTO_NACIONAL = 'N';
    protected $BASEURL;

	protected $entityManager;
	protected $util;
	protected $databaseUtil;
	protected $impressaoNFCE;
	protected $instanceManager;
	protected $systemPath;
	
	private $CDSENHACERTIF;
	private $CDINSCESTA;
	
	protected $config;
	protected $DSDIRCERTIFICADO;
	protected $CDSENHACERTIFCRIPT;
	protected $CDFILIALCERTIF;
	protected $DTEMISSAO;

    public function __construct(
    	\Doctrine\ORM\EntityManager $entityManager,
		\Odhen\API\Util\Util $util,
		\Odhen\API\Util\Database $databaseUtil,
		\Odhen\API\Service\ImpressaoNFCE $impressaoNFCE,
		\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, $BASEURL) {

	   	$this->entityManager   = $entityManager;
	   	$this->util 		   = $util;
	   	$this->databaseUtil    = $databaseUtil;
		$this->impressaoNFCE   = $impressaoNFCE;
		$this->instanceManager = $instanceManager;
		$this->systemPath      = $this->instanceManager->getParameter('SYSTEM_PATH');
		$this->nfcePath        = $this->instanceManager->getParameter('NFCE_PATH');
		$this->PROXY_IP        = $this->instanceManager->getParameter('PROXY_IP');
		$this->PROXY_PORT      = $this->instanceManager->getParameter('PROXY_PORT');
		$this->PROXY_USER      = $this->instanceManager->getParameter('PROXY_USER');
		$this->PROXY_PASSWORD  = $this->instanceManager->getParameter('PROXY_PASSWORD');
		$this->CDINSCESTA      = '';
		$this->CDSENHACERTIF   = '';
		$this->BASEURL = $BASEURL;
	}

	private function validaDadosEmpresa($cdfilial, $nrorg){
	    $getEmpresaParams = array(
	        'CDFILIAL' => $cdfilial,
	        'NRORG'    => $nrorg
    	);
		$result = $this->entityManager->getConnection()->fetchAll(self::GET_EMPRESAFILIAL,$getEmpresaParams);
		$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosEmpresa'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados para a filial informada";
		}
		return $arrayResult;
	}
	
	private function validaDadosEmpresaNew($empresa){
		$arrayResult = array();
		//CDINSCESTA, CDEMPRESA, CDPAIS, SGESTADO, CDMUNICIPIO, NRINSJURFILI, NMRAZSOCEMPR
		if(!empty($empresa['CDINSCESTA']) 
			|| 	!empty($empresa['CDEMPRESA'])
			|| !empty($empresa['CDPAIS'])
			|| !empty($empresa['SGESTADO'])
			|| !empty($empresa['CDMUNICIPIO'])
			|| !empty($empresa['NRINSJURFILI'])
			|| !empty($empresa['NMRAZSOCEMPR'])
		){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados para a filial informada";
		}
		return $arrayResult;
	}

    public function validaEnderecoFilial($cdfilial, $nrorg){
 		$getEmpresaParams = array(
	        'CDFILIAL' => $cdfilial,
	        'NRORG'    => $nrorg
    	);
    	$result = $this->entityManager->getConnection()->fetchAll(self::GET_ENDEFILI,$getEmpresaParams);
    	$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosEndereco'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há endereco para a filial logada";
		}
		return $arrayResult;
    }
    
    public function validaEnderecoFilialNew($endereco){
    	$arrayResult = array();
		if(!empty($endereco)){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há endereco para a filial logada";
		}
		return $arrayResult;
    }

    public function validaDadosEmitenteXML($cdfilial, $nrorg){
		$dadosEmitenteParams = array(
	        'CDFILIAL' => $cdfilial,
			'NRORG'    => $nrorg
	  	);
    	$result = $this->entityManager->getConnection()->fetchAssoc(self::GET_DADOSEMITENTE_IDE_XML,$dadosEmitenteParams);
    	$arrayResult = array();
		if(!empty($result)){
			$arrayResult['error'] = false;
			$arrayResult['dadosEmitente'] = $result;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados de emitente da filial.";
		}
		return $arrayResult;
    }
    
    public function validaDadosEmitenteXMLNew($emitente){
    	$arrayResult = array();
		if(!empty($emitente)){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados de emitente da filial.";
		}
		return $arrayResult;
    }

    private function validaSerieNFCE($filialInfo){

    	$serieNfceParams = array(
	        'CDFILIAL' => $filialInfo['CDFILIAL'],
			'CDCAIXA'  => $filialInfo['CDCAIXA'],
			'NRORG'    => $filialInfo['NRORG']
		);
		$result = $this->entityManager->getConnection()->fetchAll(self::GET_SERIE_NFCE,$serieNfceParams);

		$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosSerie'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há serie NFCE no caixa logado.";
		}
		return $arrayResult;
    }
    
    private function validaSerieNFCENew($serieNFCE){
		$arrayResult = array();
		if(!empty($serieNFCE)){
			$arrayResult['error'] = false;
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há serie NFCE no caixa logado.";
		}
		return $arrayResult;
    }

    private function montaArrayFilial($dadosEmitente, $dadosEmpresaResult, $endeFiliResult, $nrorg, $cdcaixa, $cdfilial, $cdloja, $cdoperador){
    	$filialInfo = array(
            'CDPRODTAXSER' => $dadosEmitente['CDPRODTAXSER'],
    		'NMRAZSOCFILI' => $dadosEmitente['NMRAZSOCFILI'],
		    'NMFILIAL' => $dadosEmitente['NMFILIAL'],
		    'NRINSJURFILI' => $this->noSpecials($dadosEmitente['NRINSJURFILI']),
		    'CDESTADOIBGE' => $dadosEmitente['CDESTADOIBGE'],
		    'CDMUNICIBGE' => strlen($dadosEmitente['CDMUNICIBGE']) != 5 ? $dadosEmitente['CDMUNICIBGE'] : $dadosEmitente['CDESTADOIBGE'] . $dadosEmitente['CDMUNICIBGE'],
		    'CDINSCESTAFILI' => $this->noSpecials($dadosEmitente['CDINSCESTA']),
		    'CDINSCMUNIFILI' => $dadosEmitente['CDINSCMUNI'],
		    'DSENDEFILI' => $dadosEmitente['DSENDEFILI'],
		    'NRENDEFILISEP' => $dadosEmitente['NRENDEFILISEP'] ? $dadosEmitente['NRENDEFILISEP'] : '.',
		    'NMBAIRFILI' => $dadosEmitente['NMBAIRFILI'],
		    'NMMUNICIPIO' => $dadosEmitente['NMMUNICIPIO'],
		    'SGESTADOFILI' => $dadosEmitente['SGESTADO'],
		    'NRCEPFILI' => str_replace('-','',$dadosEmitente['NRCEPFILI']),
		    'CDPAISBACEN' => $dadosEmitente['CDPAISBACEN'],
		    'NMPAIS' => $dadosEmitente['NMPAIS'],
		    'CDCNAE' => $dadosEmitente['CDCNAE'],
		    'CDSITUCRT'  => $dadosEmitente['CDSITUCRT'],
		    'CDEMPRESA' => $dadosEmpresaResult['CDEMPRESA'],
		    'CDINSCESTA' => $dadosEmpresaResult['CDINSCESTA'],
		    'CDPAIS' => $endeFiliResult['CDPAIS'],
		    'SGESTADO' 	=> $endeFiliResult['SGESTADO'],
			'CDMUNICIPIO' => $endeFiliResult['CDMUNICIPIO'],
			'CDFILIAL' => $cdfilial,
			'CDCAIXA' => $cdcaixa,
			'NRORG' => $nrorg,
			'CDLOJA' => $cdloja,
			'CDOPERADOR' => $cdoperador,
			'CDURLWSNFC' => $dadosEmitente['CDURLWSNFC']
    	);
    	return $filialInfo;
    }
    
    private function montaArrayFilialNew($dadosEmitente){
    	$filialInfo = array(
    		'NMRAZSOCFILI' => $dadosEmitente['NMRAZSOCFILI'],
		    'NMFILIAL' => $dadosEmitente['NMFILIAL'],
		    'NRINSJURFILI' => $this->noSpecials($dadosEmitente['NRINSJURFILI']),
		    'CDESTADOIBGE' => $dadosEmitente['CDESTADOIBGE'],
		    'CDMUNICIBGE' => strlen($dadosEmitente['CDMUNICIBGE']) != 5 ? $dadosEmitente['CDMUNICIBGE'] : $dadosEmitente['CDESTADOIBGE'] . $dadosEmitente['CDMUNICIBGE'],
		    'CDINSCESTAFILI' => $this->noSpecials($dadosEmitente['CDINSCESTA']),
		    'CDINSCMUNIFILI' => $dadosEmitente['CDINSCMUNI'],
		    'DSENDEFILI' => $dadosEmitente['DSENDEIEST'],
		    'NRENDEFILISEP' => '000',
		    'NMBAIRFILI' => $dadosEmitente['NMBAIRIEST'],
		    'NMMUNICIPIO' => $dadosEmitente['NMMUNICIPIO'],
		    'SGESTADOFILI' => $dadosEmitente['SGESTADO'],
		    'NRCEPFILI' => str_replace('-','',$dadosEmitente['NRCEPIEST']),
		    'CDPAISBACEN' => $dadosEmitente['CDPAISBACEN'],
		    'NMPAIS' => $dadosEmitente['NMPAIS'],
		    'CDCNAE' => $dadosEmitente['CDCNAE'],
		    'CDSITUCRT'  => $dadosEmitente['CDSITUCRT'],
		    'CDEMPRESA' => $dadosEmitente['CDEMPRESA'],
		    'CDINSCESTA' => $dadosEmitente['CDINSCESTA'],
		    'CDPAIS' => $dadosEmitente['CDPAIS'],
		    'SGESTADO' 	=> $dadosEmitente['SGESTADO'],
			'CDMUNICIPIO' => $dadosEmitente['CDMUNICIPIO'],
			'CDFILIAL' => $dadosEmitente['CDFILIAL'],
			'CDCAIXA' => $dadosEmitente['CDCAIXA'],
			'NRORG' => $dadosEmitente['NRORG'],
			'CDLOJA' => $dadosEmitente['CDLOJA'],
			'CDOPERADOR' => $dadosEmitente['CDOPERADOR'],
			'CDURLWSNFC' => $dadosEmitente['CDURLWSNFC']
    	);
    	return $filialInfo;
    }

    private function montaArrayInfoNFCE($dadosEmitente, $nrseqvenda, $serieNFE, $filialInfo, $connection, $VRTOTTRIBIBPT, $TOTALVENDA, $IDAMBTRABNFCE, $NRNOTAFISCALCE, $DTEMISSAOVENDA, $nrlanctonfce){
		// define informações do certificado para produção/homologação
		
		if ($dadosEmitente['IDAMBTRABNFCE'] == '1'){
			$CSC =  $dadosEmitente['CDCODSCONSPROD'];
			$CSCid = $dadosEmitente['CDIDTOKENPROD'];
		} else {
			$CSC = $dadosEmitente['CDCODSCONSHOMO'];
			$CSCid = $dadosEmitente['CDIDTOKENHOMO'];
		}

	    $serieCH = str_pad($serieNFE,3,"0",STR_PAD_LEFT);
	    //talvez necessario para diferenciar das que foram em contigencia na SEFAZ - possivel geracao de duplicidade
        //$randomNrNFE = date('d') . substr($nrseqvenda, 2);
        //$randomNrNFE = $nrseqvenda;
        //gera um numero randomico
        $randomNrNFE = rand();
        if(strlen($randomNrNFE) > 10){
        	$randomNrNFE = substr($randomNrNFE, 0, 10);
        }else{
            $randomNrNFE = str_pad($randomNrNFE,10,"0",STR_PAD_LEFT);
        }
        
        /*
        > 8 pega so os 8 ultimo
        < 8 preenche zero a esquerda
        = 8 os oitos :)
        */
        $nrlanctonfce = $this->formatValueNrlanctonfce($nrlanctonfce);
        
        
	    // $nrNFE = self::gerarNumeroNota($filialInfo, $serieCH, $connection, $IDAMBTRABNFCE);
	    $nrNFE = $NRNOTAFISCALCE;
	    $tipEmissao = $this->getPermiteContingencia($filialInfo) ? self::AMB_EMISSAO_OFFLINE : self::AMB_EMISSAO_ONLINE;
		$nrAcessoNFCE = self::gerarChaveAcessoNFCe($filialInfo, $serieCH, $nrNFE, $tipEmissao, $randomNrNFE, $DTEMISSAOVENDA, $nrlanctonfce);
		
		$nfceInfo = array(
			'CSC' => $CSC,
			'CSCid' => $CSCid,
		    'NMARQCERTNFCE' => $dadosEmitente['NMARQCERTNFCE'],
		    'DSSENHACERTNFCE' => $dadosEmitente['DSSENHACERTNFCE'],
			'tpEmis' => $tipEmissao,
			'nrNFE' => $nrNFE,
			'nrNFEInt' => intval($nrNFE),
			'randomNrNFE' => $randomNrNFE,
			'serieCH' => $serieCH,
			'cDV' => substr($nrAcessoNFCE, -1),
			'nrAcessoNFCE' => $nrAcessoNFCE,
			'VRTOTTRIBIBPT' => $VRTOTTRIBIBPT,
			'nrseqvenda' => $nrseqvenda,
			'totalVenda' => $TOTALVENDA,
			'tpAmb' => $IDAMBTRABNFCE
		);

		return $nfceInfo;
	}
	
    private function montaArrayInfoNFCENew($dadosEmitente, $filialInfo, $connection){
		// define informações do certificado para produção/homologação
		if ($dadosEmitente['IDAMBTRABNFCE'] == '1'){
			$CSC =  $dadosEmitente['CDCODSCONSPROD'];
			$CSCid = $dadosEmitente['CDIDTOKENPROD'];
		} else {
			$CSC = $dadosEmitente['CDCODSCONSHOMO'];
			$CSCid = $dadosEmitente['CDIDTOKENHOMO'];
		}
	    $serieCH = str_pad($dadosEmitente['CDSERIENFCE'],3,"0",STR_PAD_LEFT);
	    
        /*
        > 8 pega so os 8 ultimo
        < 8 preenche zero a esquerda
        = 8 os oitos :)
        */
        $nrlanctonfce = $this->formatValueNrlanctonfce($dadosEmitente['NRLANCTONFCE']);
	    $nrNFE = $dadosEmitente['NRNOTAFISCALCE'];
	    
	    $tipEmissao = $this->getPermiteContingencia($filialInfo) ? self::AMB_EMISSAO_OFFLINE : self::AMB_EMISSAO_ONLINE;
		$nrAcessoNFCE = self::gerarChaveAcessoNFCeNew($filialInfo, $serieCH, $nrNFE, $tipEmissao, $this->DTEMISSAO, $nrlanctonfce);
		
		$nfceInfo = array(
			'CSC' => $CSC,
			'CSCid' => $CSCid,
			'NMARQCERTNFCE' => $dadosEmitente['NMARQCERTNFCE'],
		    'DSSENHACERTNFCE' => $dadosEmitente['DSSENHACERTNFCE'],
			'tpEmis' => $tipEmissao,
			'nrNFE' => $nrNFE,
			'nrNFEInt' => intval($nrNFE),
			'serieCH' => intval($serieCH),
			'cDV' => substr($nrAcessoNFCE, -1),
			'nrAcessoNFCE' => $nrAcessoNFCE,
			'VRTOTTRIBIBPT' => $dadosEmitente['VRTOTTRIBIBPT'],
			'nrseqvenda' => $dadosEmitente['NRSEQVENDA'],
			'totalVenda' => $dadosEmitente['TOTALVENDA'],
			'tpAmb' => $dadosEmitente['IDAMBTRABNFCE'],
			'NRLANCTONFCE' => $dadosEmitente['NRLANCTONFCE']
		);

		return $nfceInfo;
	}

	private function gerarNumeroNota($filialInfo, $serieCH, $connection, $IDAMBTRABNFCE){
		$cdContadorNFE = 'NFCE_' . $filialInfo['CDFILIAL'] . $serieCH . '_AMB' . $IDAMBTRABNFCE;

	    return $this->util->geraCodigo($connection, $cdContadorNFE, $filialInfo['NRORG'], 1, 9);
	}

	// mudar
	public function gerarChaveAcessoNFCe($filialInfo, $serieCH, $nrNFE, $AMBEMISSAO, $randomNrNFE , $DTEMISSAOVENDA, $nrlanctonfce){
		$nrAcessoNFCE = $filialInfo['CDESTADOIBGE'] . $DTEMISSAOVENDA->format('ym') . $filialInfo['NRINSJURFILI'] .
			self::MODELO_DOC . $serieCH . $nrNFE . $AMBEMISSAO . $nrlanctonfce;
		//var_dump($nrAcessoNFCE);die;
		return $nrAcessoNFCE . self::calculaDigitoVerificadorNFe($nrAcessoNFCE);
	}
	
	public function gerarChaveAcessoNFCeNew($filialInfo, $serieCH, $nrNFE, $AMBEMISSAO, $DTEMISSAOVENDA, $nrlanctonfce){
		$nrAcessoNFCE = $filialInfo['CDESTADOIBGE'] . $DTEMISSAOVENDA->format('ym') . $filialInfo['NRINSJURFILI'] .
			self::MODELO_DOC . $serieCH . $nrNFE . $AMBEMISSAO . $nrlanctonfce;
		return $nrAcessoNFCE . self::calculaDigitoVerificadorNFe($nrAcessoNFCE);
	}

	public function validateDataNfce($nfceData){
		$result = array(
			'error' => false,
			'message' => ''
		);

		$nfceParams = self::associateCodeToMeaning();
		$missingParameters = array();
		foreach ($nfceParams as $param => $message) {
			if (empty($nfceData[$param])){
				array_push($missingParameters, $message);
			}
		}

		if (!empty($missingParameters)){
			$result['error'] = true;
			$result['message'] = 'Não foi possível utilizar a NFCE. ' . $this->nfceMissingParameterMessage($missingParameters);
		}
		return $result;
	}

	private function getSerieNFE ($serieNFCE){
		if(!$serieNFCE['dadosSerie']['CDSERIECX']){
	    	$serieNFE = intval($serieNFCE['dadosSerie']['CDCAIXA']);
	    } else {
	    	$serieNFE = intval($serieNFCE['dadosSerie']['CDSERIECX']);
	    }
	    return $serieNFE;
	}

	private function montaArraySale($ideArray, $emitArray, $infRespTec,$totalArray, $transpArray, $infAdicArray, $paymentArray, $IDAMBTRABNFCE, $infoConsumer) {
		$saleArray = array();
		$saleArray['ide'] = $ideArray;
		$saleArray['emit'] = $emitArray;
		if (!empty($infoConsumer['NRINSCRCONS'])){
	        $destArray = self::montaArrayDest($IDAMBTRABNFCE, $infoConsumer);
	        $saleArray['dest'] = $destArray;
		}
		$indexItemAprov = 0;
		foreach ($totalArray['toSaleArray'] as $item) {
			$indexItemAprov++;
			$nItem = "nItem+".$indexItemAprov;
			$saleArray['det+'.$nItem] = $item;
		}
		
		unset($totalArray['toSaleArray']);
		$saleArray['total'] = $totalArray;
		$saleArray['transp'] = $transpArray;
		$saleArray['pag'] = $paymentArray;
		$saleArray['infAdic'] = $infAdicArray;
		if(!empty($infRespTec)){
			$saleArray['infRespTec'] = $infRespTec;	
		}
		return $saleArray;
	}

	private function montaArrayPayments($filialInfo, $dtabercaix, $nrseqvenda, $totalVenda = null){
		$paymentArray = array();
		$vTroco = null;
		$vTotalPgto = null;
		$params = array(
			'CDFILIAL' => $filialInfo['CDFILIAL'],
			'CDCAIXA' => $filialInfo['CDCAIXA'],
			'NRSEQVENDA' => $nrseqvenda
		);
		$paymentModes = $this->entityManager->getConnection()->fetchAll(self::GET_PAYMENTS, $params);
		$idpag = 1;
		foreach ($paymentModes as $paymentMode) {
			if ($paymentMode['IDTIPOMOVIVE'] != 'S'){
				$paymentArray['detPag+nItem+' . $idpag] = self::getPayment($paymentMode, $filialInfo['CDFILIAL'], $filialInfo['CDCAIXA'], $dtabercaix, $nrseqvenda, $filialInfo['NRORG'], $filialInfo['CDOPERADOR']);
				$idpag++;
				$vTotalPgto = $vTotalPgto + $paymentMode['VRMOVIVEND'];
			} else {
				$vTroco = $paymentMode;
			}
		}
		if(!empty($vTroco)){
			$paymentArray['vTroco'] = !empty($vTroco) ? $this->numberFormat($vTroco['VRMOVIVEND'], 2, '.', '') : '0.00';	
		}else if (($vTotalPgto != $totalVenda) && $vTotalPgto > 0 && ($vTotalPgto > $totalVenda)){
			$paymentArray['vTroco'] = $this->numberFormat($vTotalPgto - $totalVenda, 2, '.', '');
		}else{
			$paymentArray['vTroco'] = '0.00';
		}
		return $paymentArray;
	}

	private function iniciaTransmissao($saleArray, $nfceInfo, $nfeTools, $filialInfo){
		//verifica se sera em contigencia devido ao estado
		$XMLASSINADO = self::gerarXMLValidado($saleArray, $nfeTools, $nfceInfo['nrAcessoNFCE'], $this->getPermiteContingencia($filialInfo));
		$nfeTools = json_decode($nfeTools);
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => $nfeTools),
	            array("XMLASSINADO" => $XMLASSINADO)
	        )
	    );
	    
	    $rota = 'transmitirNFCe';
	    $url  = $this->BASEURL.$rota;
	    
	    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
	    $respostaEnvio = json_decode($respostaEnvio);
	    
	    if(!empty($respostaEnvio->dataset->return[0])){
	    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
	    }
		return self::validarTransmissao($respostaEnvio, $nfeTools->config);
	}

	private function validarTransmissao($respostaEnvio, $config = null){
		$resultadoTransmissao = array(
			'XMLENVIO' => str_replace('<?xml version="1.0"?>', '', $respostaEnvio['XMLENVIO']),
			'XMLRETORNO' => $respostaEnvio['XMLRETORNO'],
			'IDSTATUSNFCE' => '',
			'DTHRPROTOCONFCE' => null,
			'NRPROTOCOLONFCE' => null,
			'message' => ''
		);
		
		
		if (!$respostaEnvio['error']){
			$respostaEnvio = (array) $respostaEnvio['data'];
			if ($respostaEnvio['cStat'] == '104'){
				$respostaEnvio = (array) $respostaEnvio['protNFe'];
				$respostaEnvio = (array) $respostaEnvio['infProt'];
				$resultadoTransmissao['IDSTATUSNFCE'] = self::getIDSTATUSNFCE($respostaEnvio['cStat']);
				$resultadoTransmissao['DTHRPROTOCONFCE'] = new \DateTime($respostaEnvio['dhRecbto']);
				$resultadoTransmissao['NRPROTOCOLONFCE'] = !empty($respostaEnvio['nProt']) ? $respostaEnvio['nProt'] : null;
			} else {
				$resultadoTransmissao['IDSTATUSNFCE'] = 'R';
				$resultadoTransmissao['DTHRPROTOCONFCE'] = new \DateTime();
			}
			$resultadoTransmissao['message'] = $respostaEnvio['xMotivo'];
		} else {
			$resultadoTransmissao['IDSTATUSNFCE'] = 'P';
			$resultadoTransmissao['message'] = $respostaEnvio['message'];
		}
		$resultadoTransmissao['message'] = $resultadoTransmissao['IDSTATUSNFCE'] . ' - ' . $resultadoTransmissao['message'];
		return $resultadoTransmissao;
	}


	//retirada essa funcao, pois os xml podem ser gerados atraves da tela Geracao de XML
	private function saveAuthorizeXML($resultadoTransmissao, $nrAcessoNFCE, $authorizePath){
		$result = array(
			'error' => false,
			'message' => ''
		);

		try {
			// junta XML assinado com retorno da SEFAZ
			$authorizeXML = Complements::toAuthorize($resultadoTransmissao['XMLENVIO'], $resultadoTransmissao['XMLRETORNO']);
			// salva no destino previamente definido
			$fpXML = fopen($authorizePath . $nrAcessoNFCE . ".xml", "w");
			fwrite($fpXML, $authorizeXML);
			fclose($fpXML);
		} catch (\Exception $e) {
			$result['error'] = true;
			$result['message'] = '<br> Erro ao salvar XML: ' . $e->getMessage();
		}

		return $result;
	}

	private function salvaXML($text) {
        //testa se a pasta existe e cria caso não exista
        $folder = $this->systemPath . "NFCE/LOGS/";
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        //--grava o log
        $line = date("d-m-Y H:i:s") . " - " . $text . "\n";
        $date = date("dmY");
        file_put_contents($this->systemPath . "NFCE/LOGS/error" . $date . ".txt", $line, FILE_APPEND);
    }

    private function getPermiteContingencia($filialInfo) {
    	if($filialInfo['SGESTADOFILI'] != 'SP' && $filialInfo['SGESTADOFILI'] != 'CE'){
    		return true;
    	}else{
    		return false;
    	}
	}

	public function transmitirNFCe($nrorg, $cdcaixa, $cdfilial, $dtabercaix, $cdloja, $cdoperador, $idtpemisvend, $nrseqvenda, $VRTOTTRIBIBPT, $TOTALVENDA, $NRNOTAFISCALCE, $simulaImpressao, $DTEMISSAOVENDA, $NRACESSONFCE, $nrlanctonfce) {
		$connection		= $this->entityManager->getConnection();
	    $idimpvenda 	= 'I';
		$DTEMISSAO = new \DateTime($DTEMISSAOVENDA);
		$this->CDFILIALCERTIF = $cdfilial;
	    // 1-> Producao, 2 -> Homologação
		$cdtipooper = '66';
        $dadosEmpresaResult = self::validaDadosEmpresa($cdfilial, $nrorg);
        if($dadosEmpresaResult['error'] == false){
            $endeFiliResult = self::validaEnderecoFilial($cdfilial, $nrorg);
            if($endeFiliResult['error'] == false){
                $dadosEmitente = self::validaDadosEmitenteXML($cdfilial, $nrorg);
                if($dadosEmitente['error'] == false){
                    $dadosEmitente = $dadosEmitente['dadosEmitente'];
            		$IDAMBTRABNFCE = $dadosEmitente['IDAMBTRABNFCE'];
				    $filialInfo = self::montaArrayFilial($dadosEmitente, $dadosEmpresaResult['dadosEmpresa'], $endeFiliResult['dadosEndereco'], $nrorg, $cdcaixa, $cdfilial, $cdloja, $cdoperador);
					$emitArray = self::montaArrayEmit($filialInfo);
				    $serieNFCE = self::validaSerieNFCE($filialInfo);
  					if($serieNFCE['error'] == false){
					    $serieNFE = self::getSerieNFE($serieNFCE);
				    	$nfceInfo = self::montaArrayInfoNFCE($dadosEmitente, $nrseqvenda, $serieNFE, $filialInfo, $connection, $VRTOTTRIBIBPT, $TOTALVENDA, $IDAMBTRABNFCE, $NRNOTAFISCALCE, $DTEMISSAO, $nrlanctonfce);
				    	$validateNfce = self::validateDataNfce(array_merge($nfceInfo, $filialInfo));
				    	if (!$validateNfce['error']){
							$productsNFCE = $this->getInfoProducts($filialInfo, $nfceInfo);
							$infoConsumer = $this->getInfoConsumer($filialInfo, $nfceInfo);
							//$DTEMISSAO = new \DateTime();
							self::createPath($IDAMBTRABNFCE, $filialInfo, $DTEMISSAO);
					        $ideArray = self::montaArrayIde($filialInfo, $nfceInfo, $serieNFE, $DTEMISSAO->format('c'), $IDAMBTRABNFCE, $nrlanctonfce);
							$totalArray = self::montaArrayTotal($productsNFCE, $filialInfo, $nrseqvenda, $cdtipooper, $IDAMBTRABNFCE, $VRTOTTRIBIBPT);
							$transpArray = self::montaArrayTransp();
							if($dadosEmitente['IDRESPTECNFCE'] == 'S'){
								$infRespTec = self::montaArrayRespTec();	
							}else{
								$infRespTec = array();
							}
							$infAdicArray = self::montaArrayInfAdic($nrseqvenda);
							$paymentArray = self::montaArrayPayments($filialInfo, $dtabercaix, $nrseqvenda);
	                        $saleArray = self::montaArraySale($ideArray, $emitArray, $infRespTec, $totalArray, $transpArray, $infAdicArray, $paymentArray, $IDAMBTRABNFCE, $infoConsumer);
	                        $nfeTools = self::getNfeTools(array_merge($nfceInfo, $filialInfo));
	                        $resultadoTransmissao = self::iniciaTransmissao($saleArray, $nfceInfo, $nfeTools, $filialInfo);
	                        $nfceInfo['IDSTATUSNFCE'] = $resultadoTransmissao['IDSTATUSNFCE'];
	                        $nfceInfo['DTHRPROTOCONFCE'] = $resultadoTransmissao['DTHRPROTOCONFCE'];
	                        $deveCancelarVenda = false;
	                        
	                        if ($resultadoTransmissao['IDSTATUSNFCE'] != 'A') {
	                        	$deveCancelarVenda = true;
	                        } else {
	                        	// salva XML de acordo com a legislação
	                        	/*$saveResult = self::saveAuthorizeXML($resultadoTransmissao, $nfceInfo['nrAcessoNFCE'], ENVIADAS_XML_PATH);
	                        	if ($saveResult['error']){
	                        		$resultadoTransmissao['message'] .= $saveResult['message'];
	                        	}*/
	                        }
	                        
	                        $resultadoTransmissao['QRCODENFCE'] = self::getQrCode($resultadoTransmissao['XMLENVIO']);
							self::updateVendaNFCE($nfceInfo, $filialInfo, $nrseqvenda, $resultadoTransmissao, $DTEMISSAO, $deveCancelarVenda);
	                        
	                        TransmissaoNfceController::escreverLog("Atualizando dados da venda.");
	                        if ($deveCancelarVenda) {
	                        	$result = array(
	                                'error' => true,
	                                'message' => 'Nota rejeitada pela SEFAZ.',
	                                'IDSTATUSNFCE' => $resultadoTransmissao['IDSTATUSNFCE'],
	                                'dadosImpressao' => array()
	                            );
	                        } else {
	                            if ($simulaImpressao) {
									$resultadoImpressao = array(
										'error' => false,
										'dadosImpressao' => array()
									);
								} else {
									$resultadoImpressao = $this->impressaoNFCE->imprimeDanfeNFCE($nfceInfo, $filialInfo, $productsNFCE, $infoConsumer, true);		
								}

								$result = array(
									'error'=> false,
									'DSQRCODE' => $resultadoTransmissao['QRCODENFCE'],
									'dadosImpressao' => array(),
									'IDSTATUSNFCE' => $resultadoTransmissao['IDSTATUSNFCE'],
									'mensagemNfce' => $resultadoTransmissao['message'],
									'mensagemImpressao' => ''
								);

								if ($resultadoImpressao['error']) {
									$result['mensagemImpressao'] = $resultadoImpressao['message'];
								} else {
									$result['dadosImpressao'] = $resultadoImpressao['dadosImpressao'];
								}
							}
							TransmissaoNfceController::escreverLog("Operação realizada.");
							return $result;
				    	} else {
				    		TransmissaoNfceController::escreverLog("Erro ao validar dados da NFCE. ". $validateNfce['message']);
				    		return $validateNfce;
				    	}
  					} else {
  						TransmissaoNfceController::escreverLog("Erro ao validar série NFCE. ".$serieNFCE['message']);
						return $serieNFCE;
					}
  				} else {
  					TransmissaoNfceController::escreverLog("Erro ao validar dados do emitente do XML. ".$dadosEmitente['message']);
					return $dadosEmitente;
				}
	    	} else {
	    		TransmissaoNfceController::escreverLog("Erro ao validar endereço da unidade. ".$endeFiliResult['message']);
				return $endeFiliResult;
			}
		} else {
			TransmissaoNfceController::escreverLog("Erro ao validar dados da empresa. ".$dadosEmpresaResult['message']);
			return $dadosEmpresaResult;
		}
	}
	
	public function transmitirNFCeNew($dadosNota) {
		$connection		= $this->entityManager->getConnection();
	    $idimpvenda 	= 'I';
	    $this->DTEMISSAO = new \DateTime($dadosNota[0]['DTEMISSAONFCE']);
		$this->CDFILIALCERTIF = $dadosNota[0]['CDFILIAL'];
	    // 1-> Producao, 2 -> Homologação
		$cdtipooper = '66';
        $dadosEmpresaResult = self::validaDadosEmpresaNew($dadosNota[0]);
        if($dadosEmpresaResult['error'] == false){
            $endeFiliResult = self::validaEnderecoFilialNew($dadosNota[0]);//
            if($endeFiliResult['error'] == false){
                $dadosEmitente = self::validaDadosEmitenteXMLNew($dadosNota[0]);
                if($dadosEmitente['error'] == false){
				    $filialInfo = self::montaArrayFilialNew($dadosNota[0]);
					$emitArray = self::montaArrayEmit($filialInfo);
				    $serieNFCE = self::validaSerieNFCENew($dadosNota[0]);
  					if($serieNFCE['error'] == false){
				    	$nfceInfo = self::montaArrayInfoNFCENew($dadosNota[0], $filialInfo, $connection);
				    	//VALIDA SE HA TODOS OS PARAMETROS
				    	$validateNfce = self::validateDataNfce(array_merge($nfceInfo, $filialInfo));
				    	if (!$validateNfce['error']){
							$infoConsumer = $this->getInfoConsumer($filialInfo, $nfceInfo);
							//RETIRANDO A GRAVACAO DO ARQUIVO NO SERVIDOR
							//self::createPath($IDAMBTRABNFCE, $filialInfo, $DTEMISSAO);
					        $ideArray = self::montaArrayIdeNew($filialInfo, $nfceInfo, $dadosNota[0]);
							$totalArray = self::montaArrayTotalNew($filialInfo, $dadosNota);
							$transpArray = self::montaArrayTransp();
							if($dadosNota[0]['IDRESPTECNFCE'] == 'S'){
								$infRespTec = self::montaArrayRespTec();	
							}else{
								$infRespTec = array();
							}
							$infAdicArray = self::montaArrayInfAdic($dadosNota[0]['NRSEQVENDA']);
							$paymentArray = self::montaArrayPayments($filialInfo, $dadosNota[0]['DTABERTUR'], $dadosNota[0]['NRSEQVENDA'], $dadosNota[0]['TOTALVENDA']);
	                        $saleArray = self::montaArraySale($ideArray, $emitArray, $infRespTec, $totalArray, $transpArray, $infAdicArray, $paymentArray, $dadosNota[0]['IDAMBTRABNFCE'], $infoConsumer);
	                        $nfeTools = self::getNfeTools(array_merge($nfceInfo, $filialInfo));
	                        $resultadoTransmissao = self::iniciaTransmissao($saleArray, $nfceInfo, $nfeTools, $filialInfo);
	                        
	                        $nfceInfo['IDSTATUSNFCE'] = $resultadoTransmissao['IDSTATUSNFCE'];
	                        $nfceInfo['DTHRPROTOCONFCE'] = $resultadoTransmissao['DTHRPROTOCONFCE'];
	                        $deveCancelarVenda = false;
	                        
	                        if ($resultadoTransmissao['IDSTATUSNFCE'] != 'A') {
	                        	$deveCancelarVenda = true;
	                        } 
	                        
	                        $resultadoTransmissao['QRCODENFCE'] = self::getQrCode($resultadoTransmissao['XMLENVIO']);
							self::updateVendaNFCE($nfceInfo, $filialInfo, $dadosNota[0]['NRSEQVENDA'], $resultadoTransmissao, $this->DTEMISSAO, $deveCancelarVenda);
	                        TransmissaoNfceController::escreverLog("Atualizando dados da venda.");
	                        if ($deveCancelarVenda) {
	                        	$result = array(
	                                'error' => true,
	                                'message' => 'Nota rejeitada pela SEFAZ.',
	                                'IDSTATUSNFCE' => $resultadoTransmissao['IDSTATUSNFCE'],
	                                'dadosImpressao' => array()
	                            );
	                        } else {
								$result = array(
									'error'=> false,
									'DSQRCODE' => $resultadoTransmissao['QRCODENFCE'],
									'dadosImpressao' => array(),
									'IDSTATUSNFCE' => $resultadoTransmissao['IDSTATUSNFCE'],
									'mensagemNfce' => $resultadoTransmissao['message'],
									'mensagemImpressao' => ''
								);
							}
							TransmissaoNfceController::escreverLog("Operação realizada.");
							return $result;
				    	} else {
				    		TransmissaoNfceController::escreverLog("Erro ao validar dados da NFCE. ". $validateNfce['message']);
				    		return $validateNfce;
				    	}
  					} else {
  						TransmissaoNfceController::escreverLog("Erro ao validar série NFCE. ".$serieNFCE['message']);
						return $serieNFCE;
					}
  				} else {
  					TransmissaoNfceController::escreverLog("Erro ao validar dados do emitente do XML. ".$dadosEmitente['message']);
					return $dadosEmitente;
				}
	    	} else {
	    		TransmissaoNfceController::escreverLog("Erro ao validar endereço da unidade. ".$endeFiliResult['message']);
				return $endeFiliResult;
			}
		} else {
			TransmissaoNfceController::escreverLog("Erro ao validar dados da empresa. ".$dadosEmpresaResult['message']);
			return $dadosEmpresaResult;
		}
	}

    public static function calculaDigitoVerificadorNFe($chave){

        $nAlgarismo = 42;
        $multiplicadores = array(2,3,4,5,6,7,8,9);
        $nMultiplicador = 0;
        $soma = 0;
        $DV = 0;

        while($nAlgarismo >= 0){
            $algarismo = substr($chave, $nAlgarismo , 1);
            $soma += $algarismo*$multiplicadores[$nMultiplicador];
            if($nMultiplicador == 7){
                $nMultiplicador = 0;
            }else{
                $nMultiplicador++;
            }
            $nAlgarismo--;
        }
        $restoDivisao = $soma % 11;
        if($restoDivisao > 1){
            $DV = 11 - $restoDivisao;
        }

        return $DV;

    }

	private function gerarXMLValidado($saleArray, $nfeTools, $nrAcessoNFCE, $contingencia){
        $unsignedXML = self::gerarXML($saleArray,$nrAcessoNFCE, $contingencia);
        // TransmissaoNfceController::escreverLog($unsignedXML);
        // TransmissaoNfceController::escreverLog('TESTE-------------------------');
        $XMLASSINADO = self::assinarXML($unsignedXML,$nrAcessoNFCE,$nfeTools);
   		return $XMLASSINADO;
	}

	private function alteraDadosParaContingencia(&$saleArray, &$nfceInfo, $filialInfo){
		$nrAcessoNFCE = self::gerarChaveAcessoNFCe($filialInfo, $nfceInfo['serieCH'], $nfceInfo['nrNFE'], self::AMB_EMISSAO_OFFLINE, $nfceInfo['randomNrNFE']);

		$nfceInfo['nrAcessoNFCE'] = $nrAcessoNFCE;
		$saleArray['ide']['cDV'] = substr($nrAcessoNFCE, -1);
		$saleArray['ide']['tpEmis'] = self::AMB_EMISSAO_OFFLINE;
	}

	private function getQrCode($signedXML){
		$begin = strpos($signedXML, '<qrCode>');
		if ($begin) {
			$beginStr = substr($signedXML, $begin + 17);
			$end = strpos($beginStr, '</qrCode>');
			$qrCode = substr($beginStr, 0, $end - 3);
		} else {
			$qrCode = "QRCode inexistente.";
		}
		return $qrCode;
	}

	private function trataMascaraImposto($valor) {
		$valor = str_replace(' ', '', $valor);
		$valor = str_replace(',', '.', $valor);
		return floatval($valor);
	}

	private function getSaleItem($cdfilial, $cdloja, $cdcaixa, $nrorg, $nrseqvenda, $nrsequitvend, $cdempresa, $cdpais, $sgestado, $cdmunicipio, $cdinscesta, $item, $cdtipooper, $CDPRODTAXSER, $VRTOTTRIBIBPTporItem) {
		
		$item_array =  self::montaArray_item();
		$cdproduto = $item['CDPRODUTO'];
		$qtprodvend = str_replace(',', '.', $item['QTPRODVEND']);
		$vrunitvend = str_replace(',', '.', $item['VRUNITVEND']);
		$nmproduto =  $item['NMPRODUTO'];

		//---- TEMPORARIO
   		$vracritvend = isset($item['VRTOTACRE']) ? $item['VRTOTACRE'] : 0;
		$vracritvend = floatval(str_replace(',', '.', $vracritvend));
   		//----
		$vrdesitvend = isset($item['VRTOTDESC']) ? $item['VRTOTDESC'] : 0;
		$vrdesitvend = floatval(str_replace(',', '.', $vrdesitvend));

        if ($cdproduto === $CDPRODTAXSER) {
            $aux = $vrunitvend;
            $vrunitvend = $vracritvend;
            $vracritvend = $aux;
        }
        
	  	$dadosProdImpParams = self::buildParamsProdImp($nrseqvenda,$nrsequitvend, $cdfilial, $cdcaixa, $cdproduto, $nrorg);
	  	$dadosProd = $this->entityManager->getConnection()->fetchAll(self::GET_DADOSPRODIMP_XML,$dadosProdImpParams);
		
		$pkCount = (is_array($dadosProd) ? count($dadosProd) : 0);
	  	if ($pkCount != 0) {
	  		$dadosProd[0]['VRALIQPIS'] = $this->trataMascaraImposto($dadosProd[0]['VRALIQPIS']);

		  	$sgunidade = $dadosProd[0]['SGUNIDADE'];
		  	$cdclasfisc = $dadosProd[0]['CDCLASFISC'];
			$cdarvprod = $dadosProd[0]['CDARVPROD'];
			// --- Dados de impostos
			$cdimposto = $dadosProd[0]['CDIMPOSTO'];
			$cstImp = $dadosProd[0]['CDCSTPRODI'];
			$cstPisCof = $dadosProd[0]['CDCSTPRODPC'];
			$vraliquotaImp = $dadosProd[0]['VRPEALPRODIT'];
			$vraliquotaPis = $dadosProd[0]['VRALIQPIS'];
			$vraliquotaCof = $dadosProd[0]['VRPERCOFINS'];
			$vrbasecalcicms = $dadosProd[0]['VRBASECALCICMS'];
			$cdCest = str_replace('.','',$dadosProd[0]['CDCEST']);
			$pRed = $dadosProd[0]['VRPERCREDUCAO'];
			$cfopImp = $dadosProd[0]['CDCFOPPFIS'];
			$cdcbenef = $dadosProd[0]['CDCBENEF']; // !!!

		  	$vUnCom = $this->numberFormat($vrunitvend, 10, '.', '');
		  	$qCom = $this->numberFormat(floatval($qtprodvend),4);

		  	$item_array['prod']['cProd'] = $cdarvprod;
		  	$item_array['prod']['cEAN'] = 'SEM GTIN';
		  	$item_array['prod']['xProd'] = preg_replace('/[^A-Za-z0-9 \-]/', '', $nmproduto);
		  	$item_array['prod']['NCM'] = $cdclasfisc;
		  	$item_array['prod']['uCom'] = $sgunidade;
		  	$item_array['prod']['qCom'] = $qtprodvend;
		  	$item_array['prod']['vUnCom'] = $vUnCom;
		  	$item_array['prod']['qCom'] = $qCom;

		  	$item_array['prod']['vProd'] = $this->numberFormat(($vUnCom * $qtprodvend),2 , '.', '');
		  	$item_array['prod']['cEANTrib'] = 'SEM GTIN';
		  	$item_array['prod']['qCom'] = $qCom;
			if($vrdesitvend > 0){
		  		$item_array['prod']['vDesc'] = $this->numberFormat($vrdesitvend, 2, '.', '');
		  	}else{
		  		unset($item_array['prod']['vDesc']);
		  	}
		  	if($vracritvend > 0){
		  		$item_array['prod']['vOutro'] = $this->numberFormat($vracritvend, 2, '.', '');
		  	}else{
		  		unset($item_array['prod']['vOutro']);
		  	}
		  	$item_array['prod']['uTrib'] = $sgunidade;
	     	$item_array['prod']['qTrib'] = $qCom;
	     	$item_array['prod']['vUnTrib'] = $vUnCom;
	     	$item_array['prod']['indTot'] = 1;//-- Compõe o total da nota

	     	$baseCalcItVend = str_replace('.', ',',(floatval(str_replace(',', '.', $qtprodvend))*floatval(str_replace(',', '.', $vrunitvend))) - $vrdesitvend + $vracritvend);
			//$baseCalcItVend = ($qtprodvend*$vrunitvend) - $vrdesitvend + $vracritvend;
			$regraImpostos = array();
			//---- TRATAMENTO DOS IMPOSTOS
			if(isset($item['IMPOSTOS'])){
				//--- impostos vindos do front
			}else{
				$orig = '0';
			   	if($cdimposto != null){
				   	$nrseqitimpos      = '001';
				   	$vrimpoprodit      = $this->valorMoeda($vraliquotaImp) * $this->valorMoeda($baseCalcItVend);
				   	$modBC             = '3';

				   	$icms_array        = self::getICMSbyCST($cstImp, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed);
				 	$pis_array 		   = self::getPISbyCST($cstPisCof, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms);
				 	$cofins_array 	   = self::getCOFINSbyCST($cstPisCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms);

				 	$tagICMS = $icms_array['TAG'];
				 	unset($icms_array['TAG']);

				 	$tagPIS = $pis_array['TAG'];
				 	unset($pis_array['TAG']);

				 	$tagCOFINS = $cofins_array['TAG'];
				 	unset($cofins_array['TAG']);

				 	$item_array['imposto']['vTotTrib'] = $VRTOTTRIBIBPTporItem;
				 	$item_array['imposto']['ICMS'][$tagICMS] = $icms_array;
				 	$item_array['imposto']['PIS'][$tagPIS] = $pis_array;
				 	$item_array['imposto']['COFINS'][$tagCOFINS] = $cofins_array;

					if(($cstImp == '10' || $cstImp == '30' || $cstImp == '60' || $cstImp == '70' || $cstImp == '90'
						|| $cstImp == '201' || $cstImp == '202' || $cstImp == '203' || $cstImp == '500' || $cstImp == '900')
						&& !empty($cdCest)){
				 		$item_array['prod']['CEST'] = $cdCest;
				 	}else{
				 		unset($item_array['prod']['CEST']);
				 	}

				 	$item_array['prod']['CFOP'] = $cfopImp;

				 	if(self::isComb($cfopImp)){
				 		$comb_array = self::montaArray_comb();
				 		if(isset($cProdANP)){
				 			$comb_array['cProdANP'] = $cProdANP;
				 			$comb_array['UFCons'] = $sgestado;
				 			$comb_array['encerrante']['nBico']   =  intval($cdbico);
   							$comb_array['encerrante']['nBomba']  =  intval($cdbomba);
   							$comb_array['encerrante']['nTanque'] =  intval($cdalmoxarifado);
   							$comb_array['encerrante']['vEncIni'] =  $this->numberFormat($vrencerranteinicial,3,'.','');
   							$comb_array['encerrante']['vEncFin'] =  $this->numberFormat($vrencerrantefinal,3,'.','');
   							$item_array['prod']['comb'] = $comb_array;
   						}
				 	}
				}
			}
	  	}
		return $item_array;
	}
	
	private function getSaleItemNew($cdfilial, $cdloja, $cdcaixa, $nrorg, $nrseqvenda, $nrsequitvend, $cdempresa, $cdpais, $sgestado, $cdmunicipio, $cdinscesta, $item, $cdtipooper, $VRTOTTRIBIBPTporItem) {
		
		$item_array =  self::montaArray_item();
		$cdproduto = $item['CDPRODUTO'];
		$qtprodvend = str_replace(',', '.', $item['QTPRODVEND']);
		$vrunitvend = str_replace(',', '.', $item['VRUNITVEND']);
		$nmproduto =  $item['NMPRODUTO'];
		
   		$vracritvend = isset($item['VRACRITVEND']) ? $item['VRACRITVEND'] : 0;
		$vracritvend = floatval(str_replace(',', '.', $vracritvend));
   		
		$vrdesitvend = isset($item['VRDESITVEND']) ? $item['VRDESITVEND'] : 0;
		$vrdesitvend = floatval(str_replace(',', '.', $vrdesitvend));
		
		$dadosProdImpParams = self::buildParamsProdImp($nrseqvenda,$nrsequitvend, $cdfilial, $cdcaixa, $cdproduto, $nrorg);
	  	$dadosProd = $this->entityManager->getConnection()->fetchAll(self::GET_DADOSPRODIMP_XML,$dadosProdImpParams);
		
		$pkCount = (is_array($dadosProd) ? count($dadosProd) : 0);
	  	if ($pkCount != 0) {
	  		$dadosProd[0]['VRALIQPIS'] = $this->trataMascaraImposto($dadosProd[0]['VRALIQPIS']);

		  	$sgunidade = $dadosProd[0]['SGUNIDADE'];
		  	$cdclasfisc = $dadosProd[0]['CDCLASFISC'];
			$cdarvprod = $dadosProd[0]['CDARVPROD'];
			// --- Dados de impostos
			$cdimposto = $dadosProd[0]['CDIMPOSTO'];
			$cstImp = $dadosProd[0]['CDCSTPRODI'];
			$cstPisCof = $dadosProd[0]['CDCSTPRODPC'];
			$vraliquotaImp = $dadosProd[0]['VRPEALPRODIT'];
			$vraliquotaPis = $dadosProd[0]['VRALIQPIS'];
			$vraliquotaCof = $dadosProd[0]['VRPERCOFINS'];
			$vrbasecalcicms = $dadosProd[0]['VRBASECALCICMS'];
			$cdCest = str_replace('.','',$dadosProd[0]['CDCEST']);
			$pRed = $dadosProd[0]['VRPERCREDUCAO'];
			$VRALIQICMSDES = $dadosProd[0]['VRALIQICMSDES'];
			$VRALIQFCP = $dadosProd[0]['VRALIQFCP'];

			$cfopImp = $dadosProd[0]['CDCFOPPFIS'];
			$cdcbenef = $dadosProd[0]['CDCBENEF']; // !!!

		  	$vUnCom = $this->numberFormat($vrunitvend, 10, '.', '');

		  	$qCom = $this->numberFormat(floatval($qtprodvend),4);

		  	$item_array['prod']['cProd'] = $cdarvprod;
		  	$item_array['prod']['cEAN'] = 'SEM GTIN';
		  	$item_array['prod']['xProd'] = preg_replace('/[^A-Za-z0-9 \-]/', '', $nmproduto);
		  	$item_array['prod']['NCM'] = $cdclasfisc;
		  	
		  	if(!empty($cdcbenef)){
		  		$item_array['prod']['cBenef'] = $cdcbenef;
		  	}else{
		  		unset($item_array['prod']['cBenef']);
		  	}
		  	
		  	$item_array['prod']['uCom'] = $sgunidade;
		  	$item_array['prod']['qCom'] = $qtprodvend;
		  	$item_array['prod']['vUnCom'] = $vUnCom;
		  	$item_array['prod']['qCom'] = $qCom;

		  	$item_array['prod']['vProd'] = $this->numberFormat(($vUnCom * $qtprodvend),2 , '.', '');
		  	$item_array['prod']['cEANTrib'] = 'SEM GTIN';
		  	$item_array['prod']['qCom'] = $qCom;
			if($vrdesitvend > 0){
		  		$item_array['prod']['vDesc'] = $this->numberFormat($vrdesitvend, 2, '.', '');
		  	}else{
		  		unset($item_array['prod']['vDesc']);
		  	}
		  	if($vracritvend > 0){
		  		$item_array['prod']['vOutro'] = $this->numberFormat($vracritvend, 2, '.', '');
		  	}else{
		  		unset($item_array['prod']['vOutro']);
		  	}
		  	$item_array['prod']['uTrib'] = $sgunidade;
	     	$item_array['prod']['qTrib'] = $qCom;
	     	$item_array['prod']['vUnTrib'] = $vUnCom;
	     	$item_array['prod']['indTot'] = 1;//-- Compõe o total da nota

	     	$baseCalcItVend = str_replace('.', ',',(floatval(str_replace(',', '.', $qtprodvend))*floatval(str_replace(',', '.', $vrunitvend))) - $vrdesitvend + $vracritvend);
			
			$orig = '0';
		   	if($cdimposto != null){
			   	$nrseqitimpos      = '001';
			   	$vrimpoprodit      = $this->valorMoeda($vraliquotaImp) * $this->valorMoeda($baseCalcItVend);
			   	$modBC             = '3';

			   	$icms_array        = self::getICMSbyCST($cstImp, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed, $VRALIQICMSDES, $VRALIQFCP, $baseCalcItVend);
			 	$pis_array 		   = self::getPISbyCST($cstPisCof, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms);
			 	$cofins_array 	   = self::getCOFINSbyCST($cstPisCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms);

			 	$tagICMS = $icms_array['TAG'];
			 	unset($icms_array['TAG']);

			 	$tagPIS = $pis_array['TAG'];
			 	unset($pis_array['TAG']);

			 	$tagCOFINS = $cofins_array['TAG'];
			 	unset($cofins_array['TAG']);

			 	$item_array['imposto']['vTotTrib'] = $VRTOTTRIBIBPTporItem;
			 	$item_array['imposto']['ICMS'][$tagICMS] = $icms_array;
			 	$item_array['imposto']['PIS'][$tagPIS] = $pis_array;
			 	$item_array['imposto']['COFINS'][$tagCOFINS] = $cofins_array;

				if(($cstImp == '10' || $cstImp == '30' || $cstImp == '60' || $cstImp == '70' || $cstImp == '90'
					|| $cstImp == '201' || $cstImp == '202' || $cstImp == '203' || $cstImp == '500' || $cstImp == '900')
					&& !empty($cdCest)){
			 		$item_array['prod']['CEST'] = $cdCest;
			 	}else{
			 		unset($item_array['prod']['CEST']);
			 	}

			 	$item_array['prod']['CFOP'] = $cfopImp;

			 	if(self::isComb($cfopImp)){
			 		$comb_array = self::montaArray_comb();
			 		if(isset($cProdANP)){
			 			$comb_array['cProdANP'] = $cProdANP;
			 			$comb_array['UFCons'] = $sgestado;
			 			$comb_array['encerrante']['nBico']   =  intval($cdbico);
   						$comb_array['encerrante']['nBomba']  =  intval($cdbomba);
   						$comb_array['encerrante']['nTanque'] =  intval($cdalmoxarifado);
   						$comb_array['encerrante']['vEncIni'] =  $this->numberFormat($vrencerranteinicial,3,'.','');
   						$comb_array['encerrante']['vEncFin'] =  $this->numberFormat($vrencerrantefinal,3,'.','');
   						$item_array['prod']['comb'] = $comb_array;
   					}
			 	}
			}
	  	}
		return $item_array;
	}

	private function buildParamsProdImp($nrseqvenda,$nrsequitvend, $cdfilial, $cdcaixa, $cdproduto, $nrorg){
		$prodImpParams = array(
			'NRSEQVENDA'   => $nrseqvenda,
			'NRSEQUITVEND' => $nrsequitvend,
			'CDFILIAL'     => $cdfilial,
			'CDCAIXA'      => $cdcaixa,
			'CDPRODUTO'    => $cdproduto,
			'NRORG'        => $nrorg
   		);
   		return $prodImpParams;
	}

	private function getPayment($payment) {
        $payment_array = self::montaArray_pag();
		$idtiporeceSEFAZ = self::getSEFAZIdtiporece($payment['IDTIPORECE']);

		$payment_array['tPag'] = $idtiporeceSEFAZ;
		$payment_array['vPag'] = $this->numberFormat($payment['VRMOVIVEND'], 2, '.', '');

		if($idtiporeceSEFAZ == '03' || $idtiporeceSEFAZ == '04'){
            $payment_array['card'] = $this->montaArray_card($payment['IDDESABTEF'], $payment['CDADMINCART'], $payment['CDBANCARTSEFAZ'], $payment['CDNSUHOSTTEF']);
        }

		return $payment_array;
	}

   	private function montaArrayIde($filialInfo, $nfceInfo, $serieNFE, $dtvendaTZ, $IDAMBTRABNFCE, $nrlanctonfce){
   		// --- IDENTIFICAÇÃO DA NOTA
		$ide =  array(
					 'cUF'    	=>  $filialInfo['CDESTADOIBGE'], // CDESTADOIBGE
			         'cNF'    	=>  substr($nrlanctonfce, -8), // NUMERO NRLANCTONFCE 8 DIG
			         'natOp'  	=>  'Descrição da natureza de operação : Venda', // DESCRIÇÃO DA NATUREZA DE OPERAÇÃO
			         'mod'   	=>  self::MODELO_DOC,
			         'serie'  	=>  $serieNFE, // CDSERIENFE
			         'nNF'	 	=>  $nfceInfo['nrNFEInt'], // NÚMERO DA NOTA (NOVOCODIGO NFCE_FILIAL_AMBIENTE)
			         'dhEmi'  	=>  $dtvendaTZ, // FULL DATETIME, MOMENTO DE EMISSÃO
			         'tpNF'   	=>  1, // TIPO DA NOTA FISCAL : 0 - ENTRADA, 1 - SAIDA
			         'idDest' 	=>  1, // ID DESTINO: 1 - INTERNO, 2 - INTERESTADUAL , 3 - EXTERIOR
			         'cMunFG' 	=>  $this->noSpecials($filialInfo['CDMUNICIBGE']), // CDMUNICIBGE
			         'tpImp'  	=>	4, // TIPO DE IMPRESSAO : 4 - DANFE NFCe
			         'tpEmis' 	=>  $nfceInfo['tpEmis'], // TIPO DE EMISSÃO : 1 - NORMAL / 9 - Contigência OFFLINE
			         'cDV'    	=>  $nfceInfo['cDV'], // DÍGITO VERIFICADOR nrAcessoNFCE
			         'tpAmb'  	=>  $IDAMBTRABNFCE, // TIPO DE AMBIENTE : 1 - PRODUÇÃO, 2 - HOMOLOGAÇÃO
			         'finNFe' 	=>  1, // FINALIDADE DA NOTA : 1 - NORMAL
			         'indFinal' =>	1, // CONSUMIDOR FINAL : 1 - SIM
			         'indPres'  =>	1, // OPERACAO PRESENCIAL : 1 - SIM
			         'procEmi'  =>	0, // PROCESSO DE EMISSÃO : 0 - POR APP DO CONTRIBUINTE
			         'verProc'	=>	'Retail Management'  // VERSÃO DO PROCESSO EMISSOR
	    );
		return $ide;
   	}
   	
   	private function montaArrayIdeNew($filialInfo, $nfceInfo, $dadosNota){
   		// --- IDENTIFICAÇÃO DA NOTA
		$ide =  array(
					 'cUF'    	=>  $filialInfo['CDESTADOIBGE'], // CDESTADOIBGE
			         'cNF'    	=>  substr($dadosNota['NRLANCTONFCE'], -8), // NUMERO NRLANCTONFCE 8 DIG
			         'natOp'  	=>  'Descrição da natureza de operação : Venda', // DESCRIÇÃO DA NATUREZA DE OPERAÇÃO
			         'mod'   	=>  self::MODELO_DOC,
			         'serie'  	=>  $nfceInfo['serieCH'], // CDSERIENFE
			         'nNF'	 	=>  $nfceInfo['nrNFEInt'], // NÚMERO DA NOTA (NOVOCODIGO NFCE_FILIAL_AMBIENTE)
			         'dhEmi'  	=>  $this->DTEMISSAO->format('c'), // FULL DATETIME, MOMENTO DE EMISSÃO
			         'tpNF'   	=>  1, // TIPO DA NOTA FISCAL : 0 - ENTRADA, 1 - SAIDA
			         'idDest' 	=>  1, // ID DESTINO: 1 - INTERNO, 2 - INTERESTADUAL , 3 - EXTERIOR
			         'cMunFG' 	=>  $this->noSpecials($filialInfo['CDMUNICIBGE']), // CDMUNICIBGE
			         'tpImp'  	=>	4, // TIPO DE IMPRESSAO : 4 - DANFE NFCe
			         'tpEmis' 	=>  $nfceInfo['tpEmis'], // TIPO DE EMISSÃO : 1 - NORMAL / 9 - Contigência OFFLINE
			         'cDV'    	=>  $nfceInfo['cDV'], // DÍGITO VERIFICADOR nrAcessoNFCE
			         'tpAmb'  	=>  $dadosNota['IDAMBTRABNFCE'], // TIPO DE AMBIENTE : 1 - PRODUÇÃO, 2 - HOMOLOGAÇÃO
			         'finNFe' 	=>  1, // FINALIDADE DA NOTA : 1 - NORMAL
			         'indFinal' =>	1, // CONSUMIDOR FINAL : 1 - SIM
			         'indPres'  =>	1, // OPERACAO PRESENCIAL : 1 - SIM
			         'procEmi'  =>	0, // PROCESSO DE EMISSÃO : 0 - POR APP DO CONTRIBUINTE
			         'verProc'	=>	'Odhen Retail'  // VERSÃO DO PROCESSO EMISSOR
	    );
		return $ide;
   	}

   	private function montaArrayEmit($filialInfo){
   		// --- IDENTIFICAÇÃO DO EMITENTE
		// - identificação do endereço do emitente
		$enderEmit = array(
						'xLgr' 	  =>	$filialInfo['DSENDEFILI'], // DSENDEFILI
						'nro'	  =>	$filialInfo['NRENDEFILISEP'], // NRENDEFILISEP
						'xBairro' =>	$filialInfo['NMBAIRFILI'], // NMBAIRFILI
						'cMun'	  =>	$filialInfo['CDMUNICIBGE'], // CDMUNICIBGE
						'xMun'	  =>	$filialInfo['NMMUNICIPIO'], // NMMUNICIPIO
						'UF'	  =>	$filialInfo['SGESTADOFILI'], // SGESTADO
						'CEP'	  =>	str_replace(array('.', '-'), '', $filialInfo['NRCEPFILI']), // CEP
						'cPais'	  =>	$filialInfo['CDPAISBACEN'], // CDPAISBACEN
						'xPais'	  =>	$filialInfo['NMPAIS']  // NMPAIS
					 );
		// - identificação do emitente
		$emit = array(
					'CNPJ'		 =>	$filialInfo['NRINSJURFILI'], // NRINSJURFILI
					'xNome'	     =>	$filialInfo['NMRAZSOCFILI'], // NMRAZSOCFILI
					'xFant'	     =>	$filialInfo['NMFILIAL'], // NMFILIAL
					'enderEmit'  => $enderEmit, // ENDERECO EMITENTE
					'IE'		 => $filialInfo['CDINSCESTAFILI'], // CDINSCESTA
					'IM'		 => $filialInfo['CDINSCMUNIFILI'], // CDINSCMUNI
					'CNAE'		 => $filialInfo['CDCNAE'], // CDCNAE
					'CRT'		 => $filialInfo['CDSITUCRT']  // CDSITUCRT
				);

		return $emit;
   	}

   	private function montaArrayDest($IDAMBTRABNFCE, $infoConsumer){
   		// --- IDENTIFICAÇÃO DO DESTINATARIO
   		$dest_array = array();

   		$cpfCNPJ = preg_replace('/[^0-9]/', '', $infoConsumer['NRINSCRCONS']);
   		if (strlen($cpfCNPJ) == 11) {
   			$dest_array['CPF'] = $cpfCNPJ;
   		} else {
   			$dest_array['CNPJ'] = $cpfCNPJ;
   		}

   		$xNome = '';
   		if ($IDAMBTRABNFCE == 1){
        	if (!empty($infoConsumer['NMCONSVEND'])){
        		$xNome = preg_replace("/&([a-z])[a-z]+;/i", "$1", trim($infoConsumer['NMCONSVEND']));
        	} else if (!empty($infoConsumer['NMCONSUMIDOR'])){
        		$xNome = preg_replace("/&([a-z])[a-z]+;/i", "$1", trim($infoConsumer['NMCONSUMIDOR']));
        	} else {
        		$xNome = 'CONSUMIDOR FINAL';
        	}
   		} else {
   			$xNome = 'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
   		}
   		$dest_array['xNome'] = $xNome;

   		if (!empty($infoConsumer['DSENDECONSVENDA']) && strlen($infoConsumer['DSENDECONSVENDA']) > 3){
   			$dest_array['enderDest'] = array(
   				'xLgr' => preg_replace("/&([a-z])[a-z]+;/i", "$1", trim($infoConsumer['DSENDECONSVENDA'])),
				'nro' => '000',
				'xBairro' => $infoConsumer['NMBAIRIEST'],
				'cMun' => strlen($infoConsumer['CDMUNICIBGE']) == 7 ? $infoConsumer['CDMUNICIBGE'] : $infoConsumer['CDESTADOIBGE'] . $infoConsumer['CDMUNICIBGE'],
				'xMun' => $infoConsumer['NMMUNICIPIO'],
				'UF' => $infoConsumer['SGESTADO'],
				'CEP' => '00000000',
				'cPais' => '1058',
				'xPais' => 'BRASIL'
   			);
   		}
   		$dest_array['indIEDest'] = '9';

		return $dest_array;
   	}

   	private function montaArray_item(){
   		// --- IDENTIFICAÇÃO DE UM ITEM DA VENDA
		// - identificação do produto que compõe o item
		$prod = array(
			'cProd'		 => null, // CDARVPROD
			'cEAN'		 => NULL, // ?????????
			'xProd'		 => null, // NMPRODUTO
			'NCM'		 => null, // CDCLASFISC
			'cBenef'	 => null, // cBenef
			'CEST'       => null, // CDCEST
			'CFOP'		 => null, // Consulta regra fiscal
			'uCom'		 => null, // SGUNIDADE
			'qCom'		 => null, // QTPRODVEND - 4 casas decimais
			'vUnCom'	 => null, // VRUNITPROD - 10 casas decimais
			'vProd'		 => null, // Valor Bruto
			'cEANTrib'	 => null, // ????????
			'uTrib'		 => null, // SGUNIDADE
			'qTrib'		 => null, // qCom
			'vUnTrib'	 => null, // vUnCom
			'vDesc'	     => null, // VRDESITVEND
			'vOutro'	 => null,
			'indTot'	 => null  // COMPÕE O TOTAL DA NOTA: 0 - NÃO; 1 - SIM
		);

		$imposto = array(
			'vTotTrib' => null,
			'ICMS'     => array(),
			'PIS'      => array(),
			'COFINS'   => array()
		);

		// - identificação do imposto de um item
		$item = array(
			'prod' 	  => $prod,
			'imposto' => $imposto
		);

		return $item;
   	}

   	private function montaArray_comb(){
   		$encerrante  = array(
   			'nBico'   =>  null,
   			'nBomba'  =>  null,
   			'nTanque' =>  null,
   			'vEncIni' =>  null,
   			'vEncFin' =>  null
   		);
   		$comb = array(
   			'cProdANP'	=> null,
   			'UFCons'	=> null,
   			'encerrante' => $encerrante
   		);
   	}

   	private function montaArray_pag(){
   		// --- IDENTIFICAÇÃO DE UM PAGAMENTO DA VENDA
		return array(
			'tPag' => null, // TIPO DE PAGAMENTO - idtiporeceSEFAZ
			'vPag' => null, // VALOR DO PAGAMENTO - VRMOVIVEND
		);
   	}

   	private function montaArray_card($IDDESABTEF, $CDADMINCART, $CDBANCARTSEFAZ, $CDNSUHOST){
   		if ($IDDESABTEF == 'N'){
	   		return array(
				'tpIntegra' => 1,
				'CNPJ' => !empty($CDADMINCART) ? preg_replace('/[^0-9]/', '', $CDADMINCART) : '00000000000000',
				'tBand' => !empty($CDBANCARTSEFAZ) ? $CDBANCARTSEFAZ : '99',
				'cAut' => !empty($CDNSUHOST) ? $CDNSUHOST :'000000000'
			);
   		} else {
   			return array(
   				'tpIntegra' => 2
   			);
   		}
   	}

   	private function montaArray_ICMS00(){
   		return array(
   			'orig' => null,
   			'CST' => '00',
   			'modBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS10(){
   		return array(
   			'orig' => null,
   			'CST' => '10',
   			'modBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS20(){
   		return array(
   			'orig' => null,
   			'CST' => '20',
   			'modBC' => null,
   			'pRedBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS30(){
   		return array(
   			'orig' => null,
   			'CST' => '30'
   		);
   	}

   	private function montaArray_ICMS40(){
		return array(
   			'orig' => null,
   			'CST' => '40'
   		);
   	}

   	private function montaArray_ICMS41(){
		return array(
   			'orig' => null,
   			'CST' => '41'
   		);
   	}

   	private function montaArray_ICMS50(){
		return array(
   			'orig' => null,
   			'CST' => '50'
   		);
   	}

   	private function montaArray_ICMS51(){
		return array(
   			'orig' => null,
   			'CST' => '51',
   			'modBC' => null,
   			'pRedBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   	}

   	private function montaArray_ICMS60(){
		return array(
   			'orig' => null,
   			'CST' => '60'
   		);
   	}

   	private function montaArray_ICMS70(){
		return array(
   			'orig'    => null,
   			'CST'     => '70',
   			'modBC'   => null,
   			'pRedBC'  => null,
   			'vBC'     => null,
   			'pICMS'   => null,
   			'vICMS'   => null,
   			'modBCST' => '3',
            'pMVAST'  => '0.00',
            'pRedBCST'=> '0.00',
            'vBCST'   => '0.00',
            'pICMSST' => '0.00',
            'vICMSST' => '0.00'
   		);
   	}

   	private function montaArray_ICMS90(){
   		return array(
   			'orig'    => null,
   			'CST'     => '90',
   			'modBC'   => null,
   			'vBC'     => null,
   			'pRedBC'  => null,
   			'pICMS'   => null,
   			'vICMS'   => null
   		);
   	}

   	private function montaArray_PISAliq(){
		return array(
   			'CST'  => null,
   			'vBC'  => null,
			'pPIS' => null,
			'vPIS' => null
   		);
   	}

   	private function montaArray_PISNT(){
		return array(
   			'CST'  => null
   		);
   	}

   	private function montaArray_PISQtde(){
		return array(
   			'CST'  => null,
   			'vBC'  => null,
			'pPIS' => null,
			'vPIS' => null
   		);
   	}

   	private function montaArray_PISOutr(){
		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pPIS' => null,
			'vPIS' => null
   		);
   	}

   	private function montaArray_COFINSAliq() {
   		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);
   	}

   	private function montaArray_COFINSQtde() {
		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);
   	}

   	private function montaArray_COFINSNT() {
   		return array(
   			'CST' => null
   		);
   	}

   	private function montaArray_COFINSOutr() {
   		return array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);
   	}

	private function montaArrayTotal($productsNFCE, $filialInfo, $nrseqvenda, $cdtipooper, $IDAMBTRABNFCE, $VRTOTTRIBIBPT) {
	
   		$icmsArray = self::montaArrayICMS();
		$totalArray = array();
		$totalArray['ICMSTot'] = $icmsArray;
	    $indexItemAprov = 0;
	    
	    $pkCount = (is_array($productsNFCE) ? count($productsNFCE) : 0);
	    $impProd = self::calcImpProd($VRTOTTRIBIBPT, $pkCount);

	    foreach ($productsNFCE as $item) {
			$nrsequitvend = $indexItemAprov + 1;
			$nrsequitvend = str_pad($nrsequitvend, 3, '0', STR_PAD_LEFT);
			$itemXml = self::getSaleItem(
				$filialInfo['CDFILIAL'],
				$filialInfo['CDLOJA'],
				$filialInfo['CDCAIXA'],
				$filialInfo['NRORG'],
				$nrseqvenda,
				$nrsequitvend,
				$filialInfo['CDEMPRESA'],
				$filialInfo['CDPAIS'],
				$filialInfo['SGESTADO'],
				$filialInfo['CDMUNICIPIO'],
				$filialInfo['CDINSCESTA'],
				$item,
				$cdtipooper,
				$filialInfo['CDPRODTAXSER'],
				$impProd['perProd']
			);
			if (!empty($itemXml['prod']['cProd'])){
				$indexItemAprov++;
				$nItem = "nItem+".$indexItemAprov;
				if($IDAMBTRABNFCE == 2 && $indexItemAprov == 1){
					$itemXml['prod']['xProd'] = 'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
				}
				// $saleArray['det+'.$nItem] = $itemXml;
				$totalArray['toSaleArray']['det+'.$nItem] = $itemXml;
				if($itemXml['prod']['indTot'] == '1'){
					$icms = $itemXml['imposto']['ICMS'];
					$totalArray = self::updateTotal($itemXml,$totalArray, $icms);
				}
			}
	    }

		$totalArray['ICMSTot']['vNF'] = self::calcTotalNF($totalArray);
		// evitar diferenças no total do imposto no caso de valor ímpar
		$totalArray['ICMSTot']['vTotTrib'] = $impProd['totTrib'];
	    return $totalArray;
   	}
   	
	private function montaArrayTotalNew($filialInfo, $dadosNota) {
	
   		$icmsArray = self::montaArrayICMS();
		$totalArray = array();
		$totalArray['ICMSTot'] = $icmsArray;
	    $indexItemAprov = 0;
	    
	    $pkCount = (is_array($dadosNota) ? count($dadosNota) : 0);
	    $impProd = self::calcImpProd($dadosNota[0]['VRTOTTRIBIBPT'], $pkCount);
	    
	    foreach ($dadosNota as $item) {
			$nrsequitvend = $indexItemAprov + 1;
			$nrsequitvend = str_pad($nrsequitvend, 3, '0', STR_PAD_LEFT);
			
			$itemXml = self::getSaleItemNew(
				$filialInfo['CDFILIAL'],
				$filialInfo['CDLOJA'],
				$filialInfo['CDCAIXA'],
				$filialInfo['NRORG'],
				$item['NRSEQVENDA'],
				$nrsequitvend,
				$filialInfo['CDEMPRESA'],
				$filialInfo['CDPAIS'],
				$filialInfo['SGESTADO'],
				$filialInfo['CDMUNICIPIO'],
				$filialInfo['CDINSCESTA'],
				$item,
				$dadosNota[0]['CDOPERADOR'],
				$impProd['perProd']
			);
			
			if (!empty($itemXml['prod']['cProd'])){
				$indexItemAprov++;
				$nItem = "nItem+".$indexItemAprov;
				if($dadosNota[0]['IDAMBTRABNFCE'] == 2 && $indexItemAprov == 1){
					$itemXml['prod']['xProd'] = 'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
				}
				// $saleArray['det+'.$nItem] = $itemXml;
				$totalArray['toSaleArray']['det+'.$nItem] = $itemXml;
				if($itemXml['prod']['indTot'] == '1'){
					$icms = $itemXml['imposto']['ICMS'];
					$totalArray = self::updateTotal($itemXml,$totalArray, $icms);
				}
			}
	    }
		$totalArray['ICMSTot']['vNF'] = self::calcTotalNF($totalArray);
		// evitar diferenças no total do imposto no caso de valor ímpar
		$totalArray['ICMSTot']['vTotTrib'] = $impProd['totTrib'];
	    return $totalArray;
   	}

   	private function calcImpProd($VRTOTTRIBIBPT, $nrProd){
   		$perProd = floatval(bcdiv($VRTOTTRIBIBPT / $nrProd, 1, 2));

   		return array(
   			'totTrib' => $this->numberFormat($perProd * $nrProd, 2, '.', ''),
   			'perProd' => $this->numberFormat($perProd, 2, '.', '')
   		);
   	}

   	private function calcTotalNF($totalArray){
   		return $this->numberFormat(round($totalArray['ICMSTot']['vProd'] - $totalArray['ICMSTot']['vDesc'] + $totalArray['ICMSTot']['vOutro'], 2), 2, '.', '');
   	}

   	private function montaArrayICMS(){
   		$icmsArray = array(
         	'vBC'		 =>	0,
         	'vICMS' 	 =>	0,
         	'vICMSDeson' =>	0,
         	'vFCP'		 => 0,
         	'vBCST' 	 =>	0,
         	'vST' 		 =>	0,
         	'vFCPST' 	 => 0,
         	'vFCPSTRet'  => 0,
    		'vProd'      =>	0,
    		'vFrete'	 =>	0,
    		'vSeg'		 =>	0,
    		'vDesc'		 =>	0,
    		'vII'		 =>	0,
    		'vIPI'		 =>	0,
    		'vIPIDevol'	 => 0,
    		'vPIS' 		 =>	0,
    		'vCOFINS'	 =>	0,
    		'vOutro'	 =>	0,
    		'vNF'		 =>	0
	    );
	    return $icmsArray;
   	}

   	private function montaArrayTransp(){
   		$transp = array(
         	'modFrete'	 =>	9
        );
        return $transp;
   	}
   	
   	private function montaArrayRespTec(){
		$infRespTec = array(
         	'CNPJ'	 =>	'26269316000177',
         	'xContato'	 =>	'Wilson Lima de Paula',
         	'email'	 =>	'qualidade@teknisa.com',
         	'fone'	 =>	'03121222300'
        );
        return $infRespTec;
   	}

   	private function montaArrayInfAdic($nrseqvenda){
   		$infAdic = array(
         	'obsCont+xCampo+NrSeqVen-IdOrigemVen' => array(
         		'xTexto' => $nrseqvenda.'-BAL_FNC'
         	)
        );
        return $infAdic;
   	}

   	private function getSEFAZIdtiporece($idtiporeceSAAS){
   		$idtiporeceSEFAZ = '99';
   		switch($idtiporeceSAAS){
   			case '1':
   				$idtiporeceSEFAZ = '03';
   				break;
   			case '2':
   				$idtiporeceSEFAZ = '04';
   				break;
   			case '3':
   				$idtiporeceSEFAZ = '02';
   				break;
   			case '4':
   				$idtiporeceSEFAZ = '01';
   				break;
   			case '5':
   				$idtiporeceSEFAZ = '11';
   				break;
   			case 'A':
   				$idtiporeceSEFAZ = '99';
   				break;
   			case '9':
   			case '8':
   				$idtiporeceSEFAZ = '05';
   				break;
   			default:
   				break;
	   	}
	   	return $idtiporeceSEFAZ;
	}

	private function getICMSbyCST($cstImp, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed, $VRALIQICMSDES = null, $VRALIQFCP = null, $baseCalcItVend = null){
		$icms_array = array();
		$tagXML = '';
		$vraliquotaImp = $vraliquotaImp/100;
		if($cstImp == '00'){
			$tagXML = 'ICMS00';
			$icms_array = self::montaArray_ICMS00();
			$icms_array['orig']  = 	$orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);

			$vICMS = (double) $vraliquotaImp * (double) $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2);
			$vICMS = $this->numberFormat($this->util->roundABNT($vICMS, 2),2, '.', '');
			$icms_array['vICMS'] = $vICMS;
			
			if(!empty($VRALIQFCP) && $VRALIQFCP > 0){
				//var_dump($VRALIQFCP);die;
				
				//doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
				$VrBaseCalc = $this->valorMoeda($baseCalcItVend);
				$VraliqfcpCalc = $this->valorMoeda($VRALIQFCP);
				
				$VrIcmsFCP = $VrBaseCalc * ($VraliqfcpCalc/100) ;
				//doVrIcmsFCP := RoundABNT(doVrBaseCalc * (cdsVendaNFCe.FieldByName('VRALIQFCP').AsFloat/100),2);
				
				$VrIcmsFCP = $this->numberFormat($this->util->roundABNT($VrIcmsFCP, 2),2, '.', ''); 

				$icms_array['pFCP']   =  $VraliqfcpCalc;
				$icms_array['vFCP']   =  $VrIcmsFCP;
				
				//var_dump($icms_array);
			}
			
		}else if($cstImp == '10'){
			$tagXML = 'ICMS10';
			$icms_array = self::montaArray_ICMS10();
			$icms_array['orig']  = 	$orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->util->roundABNT($vICMS, 2),2, '.', '');
		}else if($cstImp == '20'){
			$tagXML = 'ICMS20';
			$icms_array = self::montaArray_ICMS20();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->util->roundABNT($vICMS, 2),2, '.', '');
			
			if(!empty($VRALIQICMSDES) && $VRALIQICMSDES > 0){
				
				$VrBaseCalcDes = $this->valorMoeda($baseCalcItVend);
				$VrAliqIcmsDes = $this->valorMoeda($VRALIQICMSDES) / 100;
				$VrIcmsDes = $VrBaseCalcDes * (1 - ($VrAliqIcmsDes * (1 - ($pRed/100)) ) ) / (1 - $VrAliqIcmsDes) - $VrBaseCalcDes ;
				$VrIcmsDes = $this->numberFormat($this->util->roundABNT($VrIcmsDes, 2),2, '.', ''); 
				if($VrIcmsDes < 0.01){
					$VrIcmsDes = 0.01;
				}
				$icms_array['vICMSDeson'] = $VrIcmsDes;
				$icms_array['motDesICMS']   =  '9';
			}
			
			//var_dump($VRALIQFCP);die;
			if(!empty($VRALIQFCP) && $VRALIQFCP > 0){
				//var_dump($VRALIQFCP);die;
				
				//doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
				
				$VrBaseCalc = $this->valorMoeda($baseCalcItVend);
				$VraliqfcpCalc = ($this->valorMoeda($VRALIQFCP) * $pRed) / 100;
				
				$VrIcmsFCP = $VrBaseCalc * ($VraliqfcpCalc/100) ;
				//doVrIcmsFCP := RoundABNT(doVrBaseCalc * (cdsVendaNFCe.FieldByName('VRALIQFCP').AsFloat/100),2);
				
				$VrIcmsFCP = $this->numberFormat($this->util->roundABNT($VrIcmsFCP, 2),2, '.', ''); 

				$icms_array['vBCFCP'] =  $VrBaseCalc;
				$icms_array['pFCP']   =  $VraliqfcpCalc;
				$icms_array['vFCP']   =  $VrIcmsFCP;
			}
			
			
		}else if($cstImp == '30'){
			$tagXML = 'ICMS30';
			$icms_array = self::montaArray_ICMS30();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '40'){
			$tagXML = 'ICMS40';
			$icms_array = self::montaArray_ICMS40();
			
			if(!empty($VRALIQICMSDES) && $VRALIQICMSDES > 0){
				
				$VrBaseCalcDes = $this->valorMoeda($baseCalcItVend);
				$VrAliqIcmsDes = $this->valorMoeda($VRALIQICMSDES) / 100;
				//$VrIcmsDes = $VrBaseCalcDes * (1 - ($VrAliqIcmsDes * (1 - ($pRed/100)) ) ) / (1 - $VrAliqIcmsDes) - $VrBaseCalcDes ;
				$VrIcmsDes =   ($VrBaseCalcDes / (1 - $VrAliqIcmsDes)) * $VrAliqIcmsDes;
				$VrIcmsDes = $this->numberFormat($this->util->roundABNT($VrIcmsDes, 2),2, '.', ''); 
				if($VrIcmsDes < 0.01){
					$VrIcmsDes = 0.01;
				}
				$icms_array['vICMSDeson'] = $VrIcmsDes;
				$icms_array['motDesICMS']   =  '9';
			}
			
			$icms_array['orig']  = $orig;
		}else if($cstImp == '41'){
			$tagXML = 'ICMS40';
			$icms_array = self::montaArray_ICMS41();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '50'){
			$tagXML = 'ICMS50';
			$icms_array = self::montaArray_ICMS50();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '51'){
			$tagXML = 'ICMS51';
			$icms_array = self::montaArray_ICMS51();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = $this->numberFormat($this->util->roundABNT(substr($vICMSStr,0,$dotPos+3), 2));
		}else if($cstImp == '60'){
			$tagXML = 'ICMS60';
			$icms_array = self::montaArray_ICMS60();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '70'){
			$tagXML = 'ICMS70';
			$icms_array = self::montaArray_ICMS70();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->util->roundABNT($vICMS, 2),2, '.', '');
		}else if($cstImp == '90'){
			$tagXML = 'ICMS90';
			$icms_array = self::montaArray_ICMS90();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->util->roundABNT($vICMS, 2),2, '.', '');
			
			if(!empty($VRALIQICMSDES) && $VRALIQICMSDES > 0){
				
				$VrBaseCalcDes = $this->valorMoeda($baseCalcItVend);
				$VrAliqIcmsDes = $this->valorMoeda($VRALIQICMSDES) / 100;
				$VrIcmsDes = $VrBaseCalcDes * (1 - ($VrAliqIcmsDes * (1 - ($pRed/100)) ) ) / (1 - $VrAliqIcmsDes) - $VrBaseCalcDes ;
				$VrIcmsDes = $this->numberFormat($this->util->roundABNT($VrIcmsDes, 2),2, '.', ''); 
				if($VrIcmsDes < 0.01){
					$VrIcmsDes = 0.01;
				}
				$icms_array['vICMSDeson'] = $VrIcmsDes;
				$icms_array['motDesICMS']   =  '9';
			}
			
			if(!empty($VRALIQFCP) && $VRALIQFCP > 0){
				//var_dump($VRALIQFCP);die;
				
				//doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
				
				$VrBaseCalc = $this->valorMoeda($baseCalcItVend);
				$VraliqfcpCalc = ($this->valorMoeda($VRALIQFCP) * $pRed) / 100;
				
				$VrIcmsFCP = $VrBaseCalc * ($VraliqfcpCalc/100) ;
				//doVrIcmsFCP := RoundABNT(doVrBaseCalc * (cdsVendaNFCe.FieldByName('VRALIQFCP').AsFloat/100),2);
				
				$VrIcmsFCP = $this->numberFormat($this->util->roundABNT($VrIcmsFCP, 2),2, '.', ''); 

				$icms_array['vBCFCP'] =  $VrBaseCalc;
				$icms_array['pFCP']   =  $VraliqfcpCalc;
				$icms_array['vFCP']   =  $VrIcmsFCP;
			}
			
			
		}else if($cstImp == '102' || $cstImp == '103' || $cstImp == '300' || $cstImp == '400'){
            $tagXML = 'ICMSSN102';
            $icms_array = array();
            $icms_array['orig'] = '0';
            $icms_array['CSOSN'] = '102';
        }else if($cstImp == '500'){
            $tagXML = 'ICMSSN500';
            $icms_array = array();
            $icms_array['orig'] = '0';
			$icms_array['CSOSN'] = '500';
		}else if($cstImp == '900'){
            $tagXML = 'ICMSSN900';
            $icms_array = array();
            $icms_array['orig'] = '0';
			$icms_array['CSOSN'] = '900';
			$icms_array['modBC'] = '3';
			$icms_array['vBC']   = 	$this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pRedBC'] = $this->numberFormat(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['pICMS'] =  $this->numberFormat($vraliquotaImp*100,2);
			$vICMSStr = (string)($this->numberFormat(str_replace(',', '.',$vraliquotaImp),2) * $this->numberFormat(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $this->numberFormat($this->util->roundABNT($vICMS, 2),2, '.', '');
        }
		$icms_array['TAG'] = $tagXML;
		return $icms_array;
	}

	private function numberFormat($number, $decimals = 2, $dec_point = '.', $thousands_sep = '') {
		
		$result = 0;
		
		try {
			$result = number_format($this->trunc($number), $decimals, $dec_point, $thousands_sep);
		} catch (\Exception $e) {
			TransmissaoNfceController::escreverLog($e->getTraceAsString());
			throw new \Exception($e->getMessage().' <br> Parâmetros: number '.$number);
		}
		
		return $result;
	}

	private function trunc($val, $f = "2") {
	    if (($p = strpos($val, '.')) !== false) {
	        $val = floatval(substr($val, 0, $p + 1 + $f));
	    }
	    return $val;
	}

	private function getPISbyCST($cstPis, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms){
		$pis_array = array();
		$vraliquotaPis = $vraliquotaPis / 100;
		$vrbasecalcicms = str_replace(',', '.', $vrbasecalcicms);
		$tagXML = '';
		if ($cstPis == '01' || $cstPis == '02') {
			$pis_array = self::montaArray_PISAliq();
			$tagXML = 'PISAliq';
			$pis_array['pPIS'] = $this->numberFormat($vraliquotaPis * 100, 2);
			$pis_array['vBC']  = $this->numberFormat($vrbasecalcicms, 2, '.', '');
			$pis_array['vPIS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaPis, 2);
		} else if ($cstPis == '03') {
			$tagXML = 'PISQtde';
			$pis_array = self::montaArray_PISQtde();
			$pis_array['pPIS'] = $this->numberFormat($vraliquotaPis * 100, 2);
			$pis_array['vBC']  = $this->numberFormat($vrbasecalcicms, 2, '.', '');
			$pis_array['vPIS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaPis, 2);
		} else if ($cstPis == '04' || $cstPis == '05' || $cstPis == '06' || $cstPis == '07' || $cstPis == '08' || $cstPis == '09'){
			$tagXML = 'PISNT';
			$pis_array = self::montaArray_PISNT();
		} else if ($cstPis == '49' || $cstPis == '50' || $cstPis == '51' ||
			       $cstPis == '52' || $cstPis == '53' || $cstPis == '54' || $cstPis == '55' ||
			       $cstPis == '56' || $cstPis == '60' || $cstPis == '61' || $cstPis == '62' ||
			       $cstPis == '63' || $cstPis == '64' || $cstPis == '65' || $cstPis == '66' ||
			       $cstPis == '67' || $cstPis == '70' || $cstPis == '71' || $cstPis == '72' ||
			       $cstPis == '73' || $cstPis == '74' || $cstPis == '75' || $cstPis == '98' ||
			       $cstPis == '99') {
			$tagXML = 'PISOutr';
			$pis_array = self::montaArray_PISOutr();
			$pis_array['pPIS'] = $this->numberFormat($vraliquotaPis * 100, 2);
			$pis_array['vBC']  = $this->numberFormat($vrbasecalcicms, 2, '.', '');
			$pis_array['vPIS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaPis, 2);
		}
		$pis_array['CST'] = $cstPis;
		$pis_array['TAG'] = $tagXML;
		return $pis_array;
	}

	private function getCOFINSbyCST($cstCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms){
		$cof_array = array();
		$tagXML = '';
		$vraliquotaCof  = str_replace(',','.',$vraliquotaCof);
		$vrbasecalcicms = str_replace(',','.',$vrbasecalcicms);
		$vraliquotaCof = $vraliquotaCof/100;
		if($cstCof == '01' || $cstCof == '02'){
			$tagXML = 'COFINSAliq';
			$cof_array = self::montaArray_COFINSAliq();
			$cof_array['pCOFINS'] = $this->numberFormat($vraliquotaCof*100,2);
			$cof_array['vBC']     = $this->numberFormat($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaCof,2);
		}else if($cstCof == '03'){
			$tagXML = 'COFINSQtde';
			$cof_array = self::montaArray_COFINSQtde();
			$cof_array['pCOFINS'] = $this->numberFormat($vraliquotaCof*100,2);
			$cof_array['vBC']     = $this->numberFormat($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaCof,2);
		}else if($cstCof == '04'  || $cstCof == '05' || $cstCof == '06' || $cstCof == '07' || $cstCof == '08' || $cstCof == '09'){
			$tagXML = 'COFINSNT';
			$cof_array = self::montaArray_COFINSNT();
		}else if($cstCof == '49' || $cstCof == '50' || $cstCof == '51' ||
			     $cstCof == '52' || $cstCof == '53' || $cstCof == '54' || $cstCof == '55' ||
			     $cstCof == '56' || $cstCof == '60' || $cstCof == '61' || $cstCof == '62' ||
			     $cstCof == '63' || $cstCof == '64' || $cstCof == '65' || $cstCof == '66' ||
			     $cstCof == '67' || $cstCof == '70' || $cstCof == '71' || $cstCof == '72' ||
			     $cstCof == '73' || $cstCof == '74' || $cstCof == '75' || $cstCof == '98' ||
			     $cstCof == '99'){
			$tagXML = 'COFINSOutr';
			$cof_array = self::montaArray_COFINSOutr();
			$cof_array['pCOFINS'] = $this->numberFormat($vraliquotaCof*100,2);
			$cof_array['vBC']     = $this->numberFormat($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = $this->numberFormat($vrbasecalcicms * $vraliquotaCof,2);
		}

		$cof_array['CST'] = $cstCof;
		$cof_array['TAG'] = $tagXML;

		return $cof_array;
	}

	private function gerarXML($saleArray, $nrAcessoNFCE, $contingencia){
        $dom = self::generatesStringXMLByIndexedArray($saleArray, $nrAcessoNFCE, $contingencia);
        // trata xml
	    $XML = self::clearXmlString($dom->saveXML());
	    // salva xml
		/*$XMLName = ENTRADAS_XML_PATH . $nrAcessoNFCE.".xml";
		$handle = fopen($XMLName, "w");
		fwrite($handle, $XML);
		fclose($handle);*/
		return $XML;
	}
	
	public static function clearXmlString($string, $removeEncodingTag = false){
        $aFind = array(
            'xmlns:default="http://www.w3.org/2000/09/xmldsig#"',
            ' standalone="no"',
            'default:',
            ':default',
            "\n",
            "\r",
            "\t"
        );
        $retXml = str_replace($aFind, "", $string);
        $retXml = preg_replace('/(\>)\s*(\<)/m', '$1$2', $retXml);
        if ($removeEncodingTag) {
            $retXml = self::deleteAllBetween($retXml, '<?xml', '?>');
        }
        return $retXml;
    }
	
	public static function deleteAllBetween($string, $beginning, $end){
        $beginningPos = strpos($string, $beginning);
        $endPos = strpos($string, $end);
        if ($beginningPos === false || $endPos === false) {
            return $string;
        }
        $textToDelete = substr($string, $beginningPos, ($endPos + strlen($end)) - $beginningPos);
        return str_replace($textToDelete, '', $string);
    }

	public function createPath($IDAMBTRABNFCE, $filialInfo, $DTEMISSAO){
		$typePath = $IDAMBTRABNFCE == '2' ? 'homologacao' : 'producao';
		$XMLPath = $this->systemPath . 'NFCE' . DIRECTORY_SEPARATOR . 'NOTAS' . DIRECTORY_SEPARATOR . $typePath . DIRECTORY_SEPARATOR;
		$nfceMoment = $filialInfo['NRORG'] . '-' . $filialInfo['CDFILIAL'] . '-'
			. $filialInfo['CDCAIXA'] . '_' . $DTEMISSAO->format('Y-m-d') . DIRECTORY_SEPARATOR;

		$entradas = $XMLPath . "ENTRADAS" . DIRECTORY_SEPARATOR . $nfceMoment;
		$assinadas = $XMLPath . "ASSINADAS" . DIRECTORY_SEPARATOR . $nfceMoment;
		$enviadas = $XMLPath . "ENVIADAS" . DIRECTORY_SEPARATOR . $nfceMoment;
		$canceladas = $XMLPath . "CANCELADAS" . DIRECTORY_SEPARATOR . $nfceMoment;

		$this->util->createFolder($entradas);
		$this->util->createFolder($assinadas);
		$this->util->createFolder($enviadas);
		$this->util->createFolder($canceladas);

		if (!defined('ENTRADAS_XML_PATH')) {
		    define('ENTRADAS_XML_PATH', $entradas);
		}
		if (!defined('ASSINADAS_XML_PATH')) {
		    define('ASSINADAS_XML_PATH', $assinadas);
		}
		if (!defined('ENVIADAS_XML_PATH')) {
		    define('ENVIADAS_XML_PATH', $enviadas);
		}
		if (!defined('CANCELADAS_XML_PATH')) {
		    define('CANCELADAS_XML_PATH', $canceladas);
		}
	}

	private function getNfeTools($nfceData){
		$configContent = array(
			'atualizacao' => '2018-08-29 00:00:00',
      		'tpAmb' => intval($nfceData['tpAmb']),
      		'razaosocial' => $nfceData['NMRAZSOCFILI'],
      		'cnpj' => $nfceData['NRINSJURFILI'],
      		'ie' => $nfceData['CDINSCESTA'],
      		'siglaUF' => $nfceData['SGESTADO'],
      		'schemes' => 'PL_008i2',
      		'versao' => self::VERSAO_LAYOUT,
      		'tokenIBPT' => null,
      		'CSC' => $nfceData['CSC'],
      		'CSCid' => $nfceData['CSCid'],
      		// proxy setado aqui não funciona
      		'aProxyConf' => [
      		    'proxyIp' => null,
      		    'proxyPort' => null,
      		    'proxyUser' => null,
      		    'proxyPass' => null
      		]
		);
		
		$nfeTools = null;
		$this->config = $configContent;
		$message = null;
		
		try{
			
			if (empty($this->certContent) || $this->CDINSCESTA != $nfceData['CDINSCESTA']) {
				// lê certificado
				$this->certContent = self::getNFCeCert($nfceData);
			}
			
			//$this->DSDIRCERTIFICADO = $_SERVER['DOCUMENT_ROOT'].'/certificado.pfx';

			$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            	//array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
            	array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
            	array("config" => $this->config)),
		    );
			
		}catch (\Exception $e) {
			throw new \Exception('Dados inválidos, verifique os parâmetros nas seguintes telas: Empresas -> Inscrição Estadual. E Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT! '.self::exceptionMessage($e->getMessage()));
			//$message .= ' Erro ao tentar acessar o certificado';
			//$message = $e->getMessage();
			//throw new \Exception($message);
		}
		
		try {
			
		    $rota = 'toolsNFCe';
		    $url  = $this->BASEURL.$rota;
		    $result = $this->curlCall($url, json_encode($parameters));

		    if(!empty($result)){
		    	$result = json_decode($result);
		    	if(!isset($result->error)){
			    	if(!$result->dataset->return[0]->error){
			    		$nfeTools = $result->dataset->return[0]->ToolsNFCe;
				    }else{
				    	$message = $result->dataset->return[0]->message;
				    	throw new \Exception();
				    }
			    }else{
			    	$message = 'Erro ao acessar o link ('.$url.'). Erro: '. $result->errorCode .' - '. $result->error;
				    throw new \Exception();
			    }
		    }else{
	    		$message = 'Não houve nenhum resultado ao acessar o link ('.$url.').';
			    throw new \Exception();
		    }
		    
			// cria instância nfce
			//$nfeTools = new Tools(json_encode($configContent), Certificate::readPfx($this->certContent, $this->CDSENHACERTIF));
			
			// seta modelo
			//$nfeTools->model(self::MODELO_DOC);
			// seta o proxy
			//$nfeTools->soap->proxy($this->PROXY_IP, $this->PROXY_PORT, $this->PROXY_USER, $this->PROXY_PASSWORD);
			// seta protocolo
			//$nfeTools->soap->protocol(SoapCurl::SSL_TLSV1_2);
			
		} catch (\Exception $e) {
			throw new \Exception('Dados inválidos, verifique os parâmetros nas seguintes telas: Empresas -> Inscrição Estadual. E Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT! '.self::exceptionMessage($e->getMessage()));
			//$message .= ' '.$e->getMessage();
			//throw new \Exception($message);
		}
		
		return $nfeTools;
	}
	
	public static function exceptionMessage($msg){
		$msg = str_replace('Certificate invalid from', 'Certificado inválido. Data do certificado', $msg);
		$msg = str_replace('Unable to read certificate', 'Erro ao ler certificado', $msg);
		$msg = str_replace('Unable to open certificate', 'Erro ao abrir certificado', $msg);
		$msg = str_replace('An unexpected error has occurred when sign a content', 'Ocorreu um erro inesperado ao assinar um conteúdo', $msg);
		$msg = str_replace('An error has occurred when get private key', 'Ocorreu um erro ao obter chave privada', $msg);
		$msg = str_replace('An error has occurred when verify signature', 'Ocorreu um erro ao verificar a assinatura', $msg);
		$msg = str_replace('get follow error', 'seguinte(s) erro(s) encontrado(s)', $msg);
		return $msg;
	}

	public function getNFCeCert($nfceData){
		
		TransmissaoNfceController::escreverLog("");
		TransmissaoNfceController::escreverLog("Recuperando certificado...");
		$params[':CDINSCESTA'] = $nfceData['CDINSCESTA'];
		$params[':CDFILIAL'] = $this->CDFILIALCERTIF;
		
		$result = $this->entityManager->getConnection()->fetchAll(self::GET_LINK_CERTIFICADO, $params);
		//$result[0]['CERTIFICADO'] = '/workfolder/certificado.pfx';
		if (sizeof($result) > 0) {
			if (empty($result[0]['CERTIFICADO'])) {
				throw new \Exception('Erro, certificado não parametrizado.');
			}
			$link = $result[0]['DSDIRCERTIFICADO'];
			$this->DSDIRCERTIFICADO = $link;
			$this->CDSENHACERTIFCRIPT = $result[0]['CDSENHACERTIF'];
			$this->CDSENHACERTIF = $result[0]['DSSENHACERTNFCE']; //$this->decrypt($result[0]['CDSENHACERTIF']);
			
			TransmissaoNfceController::escreverLog("Link do certificado: ".$link);
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $link);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);

			TransmissaoNfceController::escreverLog("Estabelecendo conexão com o servidor...");
			$result = curl_exec($ch);
            $err    = curl_errno($ch);
            $errmsg = curl_error($ch);
			curl_close($ch);
			
			if ($err){
                throw new \Exception("Erro ao obter conteúdo do Certificado Digital: " . $errmsg);
            } 
            
            if (strpos($result, 'Not Found')) {
            	throw new \Exception("Erro 404. Não foi possível recuperar o certificado no servidor.". PHP_EOL . 
            	"Verificar se o Link do Servidor de Imagens encontra-se parametrizado em Paramêtros Gerais => Gerais => Geral." . PHP_EOL . 
            	"Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC0e/SAT e Empresa => Inscrição Estadual" . PHP_EOL .$errmsg);
            }
            
            //TransmissaoNfceController::escreverLog("Certificado recuperado no servidor com sucesso.");
            $this->CDINSCESTA = $nfceData['CDINSCESTA'];
            
		} else {
			throw new \Exception('Não foi possível recuperar os dados de Inscrição Estadual parametrizada.' . PHP_EOL . 
            	'Verificar se o certificado foi parametrizado em Parâmetros da Unidade => Frente de Caixa => Definições NFC0e/SAT e Empresa => Inscrição Estadual');
		}

		return $result;

	}

    public function cancelaVendaNFCE($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $venda, $DSRAZAOCANCNFCE, $CDOPERADOR){
    	$result = array(
        	'error' => true,
        	'message' => '',
        	'mensagemNfce' => '',
        	'dadosImpressao' => array()
        );
		
        try {
	    	$params = array(
				'CDFILIAL' => $CDFILIAL
			);
			$this->CDFILIALCERTIF = $CDFILIAL;
			
			$nfceData = $this->entityManager->getConnection()->fetchAssoc("GET_NFCE_DATA", $params);
			if ($nfceData['IDAMBTRABNFCE'] == '1'){
				$nfceData['CSC'] =  $nfceData['CDCODSCONSPROD'];
				$nfceData['CSCid'] = $nfceData['CDIDTOKENPROD'];
			} else {
				$nfceData['CSC'] = $nfceData['CDCODSCONSHOMO'];
				$nfceData['CSCid'] = $nfceData['CDIDTOKENHOMO'];
			}

			$validateNfce = self::validateDataNfce($nfceData);
			if (!$validateNfce['error']){
				$filialInfo = array(
					'CDFILIAL' => $CDFILIAL,
					'CDCAIXA' => $CDCAIXA,
					'NRORG' => $NRORG
				);
				self::createPath($nfceData['IDAMBTRABNFCE'], $filialInfo, new \DateTime());

				$nfceData['tpAmb'] = $nfceData['IDAMBTRABNFCE'];
				$nfeTools = self::getNfeTools($nfceData);
				// chama cancelamento
				$strNfeTools = $nfeTools->sefazCancela($venda['NRACESSONFCE'], $DSRAZAOCANCNFCE, $venda['NRPROTOCOLONFCE']);
				
				self::validarTransmissaoCancelamento($stNfeTools, $result);
				if (!$result['error']){
					$result['XMLENVIO'] = $nfeTools->lastRequest;
					$result['XMLRETORNO'] = $strNfeTools;

					/*$saveResult = self::saveAuthorizeXML($result, $venda['NRACESSONFCE'], CANCELADAS_XML_PATH);
					if ($saveResult['error']){
                		$resultadoTransmissao['mensagemNfce'] .= $saveResult['message'];
	                }*/
		        	$updateNfceCanResult = self::updateVendaStatusNfceCan($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $stNfeTools, $DSRAZAOCANCNFCE, $strNfeTools, $CDOPERADOR);
		        	if ($updateNfceCanResult['error']){
		        		$resultadoTransmissao['mensagemNfce'] .= '<br> Erro ao salvar informações do cancelamento no banco de dados: ' . $updateNfceCanResult['message'];
		        	}
				}
			} else {
				$result['message'] = $validateNfce['message'];
			}
 		} catch (\Exception $e) {
        	$result['message'] = $e->getMessage();
        }

		return $result;
    }

    private function validarTransmissaoCancelamento($stNfeTools, &$result){
    	
    	$cStat = $this->getParameterXML($stNfeTools, 'cStat');
    	
    	if ($cStat == '128') {
    		$stNfeTools2 = $this->getParameterXML($stNfeTools, 'retEvento'); 
    		$cStat = $this->getParameterXML($stNfeTools2, 'cStat');
    		$xMotivo = $this->getParameterXML($stNfeTools2, 'xMotivo');
    		
    		if ($cStat == '101'|| $cStat ==  '135' || $cStat == '155'){
    			$result['error'] = false;
    			$result['mensagemNfce'] = $xMotivo . '.';
    		}
    	}
		
		$xMotivo = $this->getParameterXML($stNfeTools, 'xMotivo');
		
    	if ($result['error']){
    		$result['message'] = 'Não foi possível cancelar o cupom fiscal. <br>' . $xMotivo . '.';
    	}
    }

    public function updateVendaStatusNfceCan($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $DSRAZAOCANCNFCE, $DSARQXMLCANCNFCE, $CDOPERADOR){
    	$result = array(
    		'error' => true,
    		'message' => ''
    	);

    	try {
    		$DSOBSSTATUSNFCE = null;
    		$NRPROTOCOLOCANC = null;
			$DTHRPROTOCOCANC = null;
			
			$stNfeToolsinfE['infEvento'] = $this->getParameterXML($DSARQXMLCANCNFCE,'infEvento');
			$stNfeTools = array();
			
    		if (!empty($stNfeToolsinfE['infEvento'])){
    			
    			$stNfeToolsinfEvent = $stNfeToolsinfE['infEvento'];
    			$stNfeTools['cStat'] = $this->getParameterXML($stNfeToolsinfEvent,'cStat');
    			$stNfeTools['xMotivo'] = $this->getParameterXML($stNfeToolsinfEvent,'xMotivo');
    			$stNfeTools['nProt'] = $this->getParameterXML($stNfeToolsinfEvent,'nProt');
    			$stNfeTools['dhRegEvento'] = $this->getParameterXML($stNfeToolsinfEvent,'dhRegEvento');
    			$DSOBSSTATUSNFCE = $stNfeTools['cStat'] . ' - ' . $stNfeTools['xMotivo'] . ' (NFCe Cancelada na SEFAZ)';
				$NRPROTOCOLOCANC = empty($stNfeTools['nProt']) ? ' ' : $stNfeTools['nProt'];
				$DTHRPROTOCOCANC = \DateTime::createFromFormat('Y-m-d\TH:i:sP', $stNfeTools['dhRegEvento']);
    		}

    		
    		$params = array(
    			'CDFILIAL' => $CDFILIAL,
				'CDCAIXA' => $CDCAIXA,
				'NRSEQVENDA' => $NRSEQVENDA,
				'DSOBSSTATUSNFCE' => $DSOBSSTATUSNFCE,
				'NRPROTOCOLOCANC' => $NRPROTOCOLOCANC,
				'DTHRPROTOCOCANC' => $DTHRPROTOCOCANC,
				'DSRAZAOCANCNFCE' => $DSRAZAOCANCNFCE,
				'DSARQXMLCANCNFCE' => $DSARQXMLCANCNFCE,
				'CDOPERADORCANC' => $CDOPERADOR
    		);
    		
			$type = array(
    			'DTHRPROTOCOCANC' => \Doctrine\DBAL\TypeS\Type::DATETIME
    		);
    		if (!$this->util->databaseIsOracle()){
				$type['DSARQXMLCANCNFCE'] = \Doctrine\DBAL\Types\Type::BINARY;
			}
			TransmissaoNfceController::escreverLog("Atualizando informações da venda.");
			TransmissaoNfceController::escreverLog("Status: ". $stNfeTools['xMotivo']);
        	$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_STATUS_NFCE_CAN, $params, $type);

        	$result['error'] = false;
        	$result['DSOBSSTATUSNFCE'] = $DSOBSSTATUSNFCE;
    	} catch (\Exception $e) {
    		$result['message'] = $e->getMessage();
    	}

    	return $result;
    }
    
    
    /*
    910 Rejeição: Chave de Acesso NF-e Substituta inválida (<nome do campo>)
	911 Rejeição: Chave de Acesso NF-e Substituta incorreta (<nome do campo>)
	912 Rejeição: NF-e Substituta inexistente
	913 Rejeição: NF-e Substituta Denegada ou Cancelada
	914 Rejeição: Data de emissão da NF-e Substituta maior que 2 horas da data de emissão da NF- e a ser cancelada
	915 Rejeição: Valor total da NF-e Substituta difere do valor da NF-e a ser cancelada
	916 Rejeição: Valor total do ICMS da NF-e Substituta difere do valor da NF-e a ser cancelada
	917 Rejeição: Identificação do destinatário da NF-e Substituta difere da identificação do destinatário da NF-e a ser cancelada.
	918 Rejeição: Quantidade de itens da NF-e Substituta difere da quantidade de itens da NF-e a ser cancelada.
	919 Rejeição: Item da NF-e Substituta difere do mesmo item da NF-e a ser cancelada.
	920 Rejeição: Tipo de Emissão inválido no Cancelamento por Substituição
    */
    
    
    public function updateVendaStatusNfceCanSB($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $DSRAZAOCANCNFCE, $DSARQXMLCANCNFCE, $CDOPERADOR, $OBSSTATUSNFCE){
    	$result = array(
    		'error' => true,
    		'message' => ''
    	);
    	
    	$arrayRejeicoes = ['910', '911', '912', '913', '914', '915', '916', '917', '918', '919', '920', '656'];
		
		
    	try {
    		$DSOBSSTATUSNFCE = null;
    		$NRPROTOCOLOCANC = null;
			$DTHRPROTOCOCANC = null;
			$STATUSCANCELA = 'A';
			
			$stNfeToolsinfE['infEvento'] = $this->getParameterXML($DSARQXMLCANCNFCE,'infEvento');
			$stNfeTools = array();
			
    		if (!empty($stNfeToolsinfE['infEvento'])){
    			$stNfeToolsinfEvent = $stNfeToolsinfE['infEvento'];
    			
    			
    			$stNfeTools['cStat'] = $this->getParameterXML($stNfeToolsinfEvent,'cStat');
    			$stNfeTools['xMotivo'] = $this->getParameterXML($stNfeToolsinfEvent,'xMotivo');
    			
				//verifica se nota foi realmente cancelada através do status retornado
				if (!in_array($stNfeTools['cStat'], $arrayRejeicoes)) { 
					$stNfeTools['dhRegEvento'] = $this->getParameterXML($stNfeToolsinfEvent,'dhRegEvento');
	    			$stNfeTools['nProt'] = $this->getParameterXML($stNfeToolsinfEvent,'nProt');
	    			
					$NRPROTOCOLOCANC = empty($stNfeTools['nProt']) ? ' ' : $stNfeTools['nProt'];
					$DTHRPROTOCOCANC = \DateTime::createFromFormat('Y-m-d\TH:i:sP', $stNfeTools['dhRegEvento']);
					$STATUSCANCELA = 'C';
					$DSOBSSTATUSNFCE = $stNfeTools['cStat'] . ' - ' . $stNfeTools['xMotivo'] . ' (NFCe Cancelada na SEFAZ)';
					
				}else{
					$DSOBSSTATUSNFCE = $OBSSTATUSNFCE.'! Evento de Cancelamento por Substituição: '.$stNfeTools['cStat'] . ' - ' . $stNfeTools['xMotivo'];
				}
				
				$params = array(
	    			'CDFILIAL' => $CDFILIAL,
					'CDCAIXA' => $CDCAIXA,
					'NRSEQVENDA' => $NRSEQVENDA,
					'DSOBSSTATUSNFCE' => $DSOBSSTATUSNFCE,
					'NRPROTOCOLOCANC' => $NRPROTOCOLOCANC,
					'DTHRPROTOCOCANC' => $DTHRPROTOCOCANC,
					'DSRAZAOCANCNFCE' => $DSRAZAOCANCNFCE,
					'DSARQXMLCANCNFCE' => $DSARQXMLCANCNFCE,
					'CDOPERADORCANC' => $CDOPERADOR,
					'IDSTATUSNFCE' => $STATUSCANCELA
	    		);
	    		
	    		$type = array(
	    			'DTHRPROTOCOCANC' => \Doctrine\DBAL\TypeS\Type::DATETIME
	    		);	
			
	    		if (!$this->util->databaseIsOracle()){
					$type['DSARQXMLCANCNFCE'] = \Doctrine\DBAL\Types\Type::BINARY;
				}
				
				TransmissaoNfceController::escreverLog("Atualizando informações da venda.");
				TransmissaoNfceController::escreverLog("Status: ". $stNfeTools['xMotivo']);
	        	$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_STATUS_NFCE_CAN_SB, $params, $type);
	
	        	$result['error'] = false;
	        	$result['DSOBSSTATUSNFCE'] = $DSOBSSTATUSNFCE;
				
    		}else{
    			
    			$result['error'] = true;
        		$result['DSOBSSTATUSNFCE'] = "Erro ao obter informações do evento!";
    		}

    		
    		
    	} catch (\Exception $e) {
    		$result['message'] = $e->getMessage();
    	}

    	return $result;
    }
    
    private function recuperarNfeTools($nota) {
    	if ($nota['IDTPAMBNFCE'] == '1'){
			$CSC =  isset($nota['CDCODSCONSPROD']) ? $nota['CDCODSCONSPROD'] : null;
			$CSCid = isset($nota['CDIDTOKENPROD']) ? $nota['CDIDTOKENPROD'] : null;
		} else {
			$CSC = isset($nota['CDCODSCONSHOMO']) ? $nota['CDCODSCONSHOMO'] : null;
			$CSCid = isset($nota['CDIDTOKENHOMO']) ? $nota['CDIDTOKENHOMO'] : null;
		}
		
		$this->CDFILIALCERTIF = $nota['CDFILIAL'];
		
		$parametros = array (
			'tpAmb' 	      => $nota['IDTPAMBNFCE'],
			'NMRAZSOCFILI'    => $nota['NMRAZSOCFILI'],
			'NRINSJURFILI'    => $nota['NRINSJURFILI'],
			'CDINSCESTA'      => $nota['CDINSCESTA'],
			'SGESTADO'        => $nota['SGESTADO'],
			'CSC'		      => $CSC,
			'CSCid'		      => $CSCid,
			'CDURLWSNFC'      => $nota['CDURLWSNFC'],
			'NMARQCERTNFCE'   => '',
			'DSSENHACERTNFCE' => $this->CDSENHACERTIF
		);
	    return self::getNfeTools($parametros);
    }
    
    public function assinarXMLExterno($xml, $nota) {
		$nfeTools = $this->recuperarNfeTools($nota);
		//$XMLASSINADO = $nfeTools->signNFe($xml);
		//resolver essa situacao
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("unsignedXML" => $xml)
	        )
	    );
	    
	    $rota = 'signNFe';
	    $url  = $this->BASEURL.$rota;
	    $XMLASSINADO = $this->curlCall($url, json_encode($parameters));
	    if(!empty($XMLASSINADO)){
	    	$XMLASSINADO = json_decode($XMLASSINADO);
	    	if(!isset($XMLASSINADO->error)){
	    		if(!$XMLASSINADO->dataset->return[0]->error){
			    	$XMLASSINADO = $XMLASSINADO->dataset->return[0]->XMLASSINADO;
					return $XMLASSINADO;
			    }else{
			    	return $XMLASSINADO->dataset->return[0]->message;
			    }
	    	}else{
	    		return 'Erro ao tentar assinar XML. Erro:' .$XMLASSINADO->errorCode .' - '.$XMLASSINADO->error;
	    	}
		    
	    }else{
	    	return 'Não houve nenhum retorno ao tentar assinar XML pelo link ('.$url.')';
	    }
	    
		//$XMLASSINADO = $this->assinarXML($xml, $chaveAcesso, $nfeTools);
    	//return $XMLASSINADO;
    }
    
    public function invalidarNota($nota, $parametros) {
    	$nfeTools = $this->recuperarNfeTools($nota);
    	
    	$nSerie = (int) $nota['CDSERIENFCE'];
    	$nIni   = (int) $nota['NRNOTAFISCALCE'];
    	$nFin   = (int) $nota['NRNOTAFISCALCE'];
    	$xJust  = $parametros['DSOBSMOVBENECONS'];
    	$tpAmb  = $nota['IDAMBTRABNFCE'];
    	
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("nIni" => $nIni),
	            array("nFin" => $nFin),
	            array("xJust" => $xJust),
	            array("tpAmb" => $tpAmb),
	            array("nSerie" => $nSerie)
	        )
	    );
	    
	    
	    $rota = 'inutilizaNFCe';
	    $url  = $this->BASEURL.$rota;
	    $result = '';
	    
	    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
	    if(!empty($respostaEnvio)){
	    	$respostaEnvio = json_decode($respostaEnvio);
	    	if(!$respostaEnvio->dataset->return[0]->error){
				return $respostaEnvio->dataset->return[0]->XMLRETORNO;
		    }else{
		    	return $respostaEnvio->dataset->return[0]->message;
		    }	
	    }else{
	    	return 'Erro ao Inutilizar ';	
	    }
    	//return $nfeTools->sefazInutiliza($nSerie, $nIni, $nFin, $xJust, $tpAmb);
    }

	private function assinarXML($unsignedXML, $nrAcessoNFCE, $nfeTools){
		//$signedXMLName = ASSINADAS_XML_PATH . $nrAcessoNFCE . ".xml";
		
		//resolver essa situacao
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("unsignedXML" => $unsignedXML)
	        )
	    );
	    
	    $rota = 'signNFe';
	    $url  = $this->BASEURL.$rota;
	    $XMLASSINADO = $this->curlCall($url, json_encode($parameters));
	    $XMLASSINADO = json_decode($XMLASSINADO);
	    
	    if(!$XMLASSINADO->dataset->return[0]->error){
	    	$XMLASSINADO = $XMLASSINADO->dataset->return[0]->XMLASSINADO;
			//$XMLASSINADO = $nfeTools->signNFe($unsignedXML);
			/*$sHandle = fopen($signedXMLName, "w");
			fwrite($sHandle, $XMLASSINADO);
			fclose($sHandle);*/
			return $XMLASSINADO;
	    }else{
	    	return $XMLASSINADO->dataset->return[0]->message;
	    }
	}

	protected function transmitirXML($nfeTools, $XMLASSINADO) {
		$return = array(
			'error' => true,
			'data' => array(),
			'message' => '',
			'XMLENVIO' => $XMLASSINADO,
			'XMLRETORNO' => ''
		);
		$idLote = substr(str_replace(',', '', number_format(microtime(true)*1000000, 0)), 0, 15);

		try {
			$strNfeTools = $nfeTools->sefazEnviaLote(array($XMLASSINADO), $idLote, 1);
			$return['XMLRETORNO'] = $strNfeTools;
			$stNfeTools = new Standardize($strNfeTools);
			$return['data'] = $stNfeTools->toArray();
			$return['error'] = false;
		} catch(\Exception $e) {
			$return['message'] = $e->getMessage();
		}

		return $return;
	}

	private function generatesStringXMLByIndexedArray($saleArray, $nrAcessoNFCE, $contingencia){
		$dom = new \DOMDocument('1.0', 'utf-8');
		//------ Atributos
		$xmlns = $dom->createAttribute('xmlns');
		$xmlns->value = 'http://www.portalfiscal.inf.br/nfe';

        //------ NFe
        $NFe = $dom->createElement('NFe');
        $NFe->appendChild($xmlns);
        $dom->appendChild($NFe);

        //------ infNFe
        $infNFe = $dom->createElement('infNFe');
        $Id = $dom->createAttribute('Id');
        $infNFeVersao = $dom->createAttribute('versao');
        $Id->value = 'NFe'.$nrAcessoNFCE;
        $infNFeVersao->value = self::VERSAO_LAYOUT;

		$infNFe->appendChild($Id);
		$infNFe->appendChild($infNFeVersao);

		$NFe->appendChild($infNFe);

        if($contingencia){
            $saleArray['ide']['dhCont'] = $saleArray['ide']['dhEmi'];
            $saleArray['ide']['xJust'] = 'Motivo Emissao em Contingencia: Caixa OffLine';
            $saleArray['infAdic']['infCpl'] = 'NFCe EMITIDA EM CONTINGENCIA';
            ksort($saleArray['infAdic']);
        }
        self::arrayElementsToXML($saleArray,$infNFe,$dom);

     	return $dom;

	}

	private function arrayElementsToXML($array,&$currentTag,&$dom){
		foreach($array as $key=>$value){
			$newElement =  null;
			$keyAsArray = explode('+',$key);
			
			$pkCount = (is_array($keyAsArray) ? count($keyAsArray) : 0);
			if($pkCount > 1){
				$key = $keyAsArray[0];
				$newElement = $dom->createElement($key);
				if($key == 'det' || $key == 'obsCont'){
					$attrName = $keyAsArray[1];
					$attrValue = $keyAsArray[2];
					$attribute = $dom->createAttribute($attrName);
					$attribute->value = $attrValue;
					$newElement->appendChild($attribute);
				}
				$currentTag->appendChild($newElement);
				if(is_array($value)){
					self::arrayElementsToXML($value,$newElement,$dom);
				}else{
					$newElement->nodeValue = $value;
				}
			}else{
				if(is_array($value)){
					$newElement = $dom->createElement($key);
					$currentTag->appendChild($newElement);
					self::arrayElementsToXML($value,$newElement,$dom);
				}else{
					$newElement = $dom->createElement($key,$value);
					$currentTag->appendChild($newElement);
				}
			}

        }
	}

	private function updateVendaNFCE($nfceInfo, $filialInfo, $nrseqvenda, $resultadoTransmissao, $DTEMISSAO, $deveCancelarVenda){
		$params = array(
			'CDSERIENFCE' => $nfceInfo['serieCH'],
			'NRACESSONFCE' => $nfceInfo['nrAcessoNFCE'],
			'NRNOTAFISCALCE' => $nfceInfo['nrNFE'],
			'NRLANCTONFCE' => $nfceInfo['NRLANCTONFCE'],
			'IDTPAMBNFCE' => $nfceInfo['tpAmb'],
			'IDSTATUSNFCE' => $resultadoTransmissao['IDSTATUSNFCE'],
			'DTEMISSAONFCE' => $DTEMISSAO,
			'NRPROTOCOLONFCE' => $resultadoTransmissao['NRPROTOCOLONFCE'],
			'DSOBSSTATUSNFCE' => $resultadoTransmissao['message'],
			'IDMODOPERACNFCE' => '1',
			'NRRECIBONFCE' => null,
			'CDOPERENVIONFCE' => null,
			'DTHRPROTOCONFCE' => $resultadoTransmissao['DTHRPROTOCONFCE'],
			'IDXMLVALIDNFCE' => 'S',
			'DSQRCODENFCE' => $resultadoTransmissao['QRCODENFCE'],
			'DSARQXMLNFCE' => $resultadoTransmissao['XMLENVIO'],
			'CDFILIAL' => $filialInfo['CDFILIAL'],
			'CDCAIXA' => $filialInfo['CDCAIXA'],
			'NRSEQVENDA' => $nrseqvenda,
			'CDVERSXMLNFCVND' => self::VERSAO_LAYOUT,
			'NRORG' => $filialInfo['NRORG']
		);
		TransmissaoNfceController::escreverLog($resultadoTransmissao['message']);
		$types = array(
			'DTEMISSAONFCE' => \Doctrine\DBAL\Types\Type::DATETIME,
			'DTHRPROTOCONFCE' => \Doctrine\DBAL\Types\Type::DATETIME
		);
		if (!$this->util->databaseIsOracle()){
			$types['DSARQXMLNFCE'] = \Doctrine\DBAL\Types\Type::BINARY;
		}

		$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_NFCE, $params, $types);
	}

	private function updateTotal($property_array,&$total_xml, $icms){
		$ICMSTot = $total_xml['ICMSTot'];
		$vBCAcumulado = $ICMSTot['vBC'];
		$ICMSTot['vBC'] = 0;
		foreach($ICMSTot as $tKey=>$totalProperty){
			self::updateTotalProperty($tKey,$totalProperty, $property_array);
			$ICMSTot[$tKey] = $this->numberFormat($totalProperty,2 ,'.', '');
		}

		$this->updateVBCTot($ICMSTot,$icms);

		$ICMSTot['vBC'] = $this->numberFormat($ICMSTot['vBC'] + $vBCAcumulado,2 ,'.', '');
		$total_xml['ICMSTot'] = $ICMSTot;
		return $total_xml;
	}

	private function updateTotalProperty($propertyName, &$propertyValue, $arrayToCompare){
		foreach($arrayToCompare as $pKey=>$item){
			if(is_array($item)){
			   self::updateTotalProperty($propertyName,$propertyValue,$item);
			}else{
				if($propertyName == $pKey && $propertyName != 'vBC'){
					$propertyValue = $propertyValue + $item;
				}
				// else if($propertyName == $pKey && $propertyName == 'vBC'){
				// 	$propertyValue = $item;
				// }
			}
		}
	}

	private function updateVBCTot(&$ICMSTot,$icms){
		$icmsArray  = array_values($icms);
		$icmsNode  = $icmsArray[0];
		if(isset($icmsNode['vBC'])){
			$ICMSTot['vBC'] += (double)$icmsNode['vBC'];
		}
	}
	
	private function formataCaracteresXML($strXML){
		$formattedXML = strtr(
	            $strXML,
     			array (
			      'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A',
      			  'Æ' => 'A', 'Ç' => 'C', 'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E',
      			  'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'Ð' => 'D', 'Ñ' => 'N',
				  'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O',
				  'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ý' => 'Y', 'Ŕ' => 'R',
				  'Þ' => 's', 'ß' => 'B', 'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a',
				  'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c', 'è' => 'e', 'é' => 'e',
				  'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i',
				  'ð' => 'o', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o',
				  'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ý' => 'y',
				  'þ' => 'b', 'ÿ' => 'y', 'ŕ' => 'r'
    			)
			  );
		return $formattedXML;
	}

	private function isComb($cfop){
		$isComb = false;
		if($cfop =='1651' || $cfop =='1652' || $cfop =='1653' || $cfop =='1658' || $cfop =='1659' ||
		   $cfop =='1660' || $cfop =='1661' || $cfop =='1662' || $cfop =='1663' || $cfop =='1664' ||
		   $cfop =='2651' || $cfop =='2652' || $cfop =='2653' || $cfop =='2658' || $cfop =='2659' ||
		   $cfop =='2660' || $cfop =='2661' || $cfop =='2662' || $cfop =='2663' || $cfop =='2664' ||
		   $cfop =='3651' || $cfop =='3652' || $cfop =='3653' || $cfop =='5651' || $cfop =='5652' ||
		   $cfop =='5653' || $cfop =='5654' || $cfop =='5655' || $cfop =='5656' || $cfop =='5657' ||
		   $cfop =='5658' || $cfop =='5659' || $cfop =='5660' || $cfop =='5661' || $cfop =='5662' ||
		   $cfop =='5663' || $cfop =='5664' || $cfop =='5665' || $cfop =='5666' || $cfop =='5667' ||
		   $cfop =='6651' || $cfop =='6652' || $cfop =='6653' || $cfop =='6654' || $cfop =='6655' ||
		   $cfop =='6656' || $cfop =='6657' || $cfop =='6658' || $cfop =='6659' || $cfop =='6660' ||
		   $cfop =='6661' || $cfop =='6662' || $cfop =='6663' || $cfop =='6664' || $cfop =='6665' ||
		   $cfop =='6666' || $cfop =='6667' || $cfop =='7651' || $cfop =='7654' || $cfop =='7667') {

				$isComb = true;
		}

		return $isComb;
	}

	private function getIDSTATUSNFCE($cStat) {
		$idStatus = '';

		switch($cStat){
			case '100':
			case '150':
				$idStatus = 'A';
				break;
			case '105':
				$idStatus = 'T';
				break;
			case '215':
				$idStatus = 'R';
			case '110':
			case '301':
			case '302':
			case '303':
			case '999':
				$idStatus = 'D';
				break;
			default:
				$idStatus = 'R';
				break;
		}
		return $idStatus;
	}

	private function nfceMissingParameterMessage($missingParameters){
		
		$pkCount = (is_array($missingParameters) ? count($missingParameters) : 0);
		$oneMissingParameter = $pkCount == 1;
		$message = $oneMissingParameter ? 'O parâmetro ' : 'Os parâmetros ';

		if($oneMissingParameter) {
			$message .= $missingParameters[0];
		} else {
			$pkCountMissing = (is_array($missingParameters) ? count($missingParameters) : 0);
			$lastIndex = $pkCountMissing - 1;
			foreach ($missingParameters as $key => $missing) {
				$message .= $missing;

				$divisor = '';
				if($key == ($lastIndex - 1)){
					$divisor = ' e ';
				} else if ($key != $lastIndex){
					$divisor = ', ';
				}
				$message .= $divisor;
			}
		}

		return $message . ' da filial estão nulos e necessitam ser parametrizados. Deve ser realizada na tela Parâmetros da Unidade -> Frente de Caixa -> Definições NFC-e / SAT';
	}

	private function associateCodeToMeaning(){
		return array(
			'NMRAZSOCFILI' => 'razão social',
			'SGESTADO' => 'estado',
			'NRINSJURFILI' => 'CNPJ',
			'CDINSCESTA' => 'Inscrição Estadual',
			'CSC' => 'token do CSC',
			'CSCid' => 'identificador do CSC',
			'NMARQCERTNFCE' => 'nome do arquivo do certificado digital',
			'DSSENHACERTNFCE' => 'senha do arquivo do certificado digital'
		);
	}

	public function getInfoProducts($filialInfo, $nfceInfo){
		$paramsToGetProducts = array(
			'CDFILIAL'	 => $filialInfo['CDFILIAL'],
			'CDCAIXA'    => $filialInfo['CDCAIXA'],
			'NRSEQVENDA' => $nfceInfo['nrseqvenda']
		);
		return $this->entityManager->getConnection()->fetchAll(self::GET_PRODUCTS_TO_PRINT_NFCE, $paramsToGetProducts);

	}

	public function getInfoConsumer($filialInfo, $nfceInfo){
		$paramsToConsumer = array(
			':CDFILIAL'   => $filialInfo['CDFILIAL'],
			':CDCAIXA'    => $filialInfo['CDCAIXA'],
			':NRSEQVENDA' => $nfceInfo['nrseqvenda']
		);
        return $this->entityManager->getConnection()->fetchAssoc(self::GET_CONSUMER_SALE, $paramsToConsumer);
	}
	
	public function decrypt($text, $salt = 'teknisa'){
        return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    public function convertResult($result){
    	$stdCl = new Standardize($result);
		$rt = $stdCl->toStd();
		
		return $rt;
    }
	
	public function consultaSituacaoNFe($nota) {
		$nfeTools = $this->recuperarNfeTools($nota);
		
		//ira consultar duas vezes, com a chave gravada no banco e com a chave alterada
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("NRACESSONFCE" => $nota['NRACESSONFCE']),
	            array("IDTPAMBNFCE" => $nota['IDTPAMBNFCE'])
	        )
	    );
	    
	    $rota = 'consultaNFCe';
	    $url  = $this->BASEURL.$rota;
	    $result = '';
	    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
	    if(!empty($respostaEnvio)){
	    	$respostaEnvio = json_decode($respostaEnvio);
	    	if(!isset($result->error)){
	    		if(!empty($respostaEnvio->dataset->return[0])){
			    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
			    	$result = $respostaEnvio['XMLRETORNO'];
			    	$status = $this->getParameterXML($result, 'cStat');
					if($status == 217 || $status == 539){
						$nChave = $this->recalculaChave($nota['NRACESSONFCE']);
						$parameters = array("requestType" => "DataSet", "dataset" => 
					        array(
				            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
					            array("nfeTools" => json_decode($nfeTools)),
					            array("NRACESSONFCE" => $nChave),
					            array("IDTPAMBNFCE" => $nota['IDTPAMBNFCE'])
					        )
					    );
					    
					    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
					    $respostaEnvio = json_decode($respostaEnvio);
					    if(!empty($respostaEnvio->dataset->return[0])){
					    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
					    	$result = $respostaEnvio['XMLRETORNO'];
					    	$status = $this->getParameterXML($result, 'cStat');
							if($status == 217 || $status == 539){
								$nChave = $this->recalculaChave($nota['NRACESSONFCE'], true);
								$parameters = array("requestType" => "DataSet", "dataset" => 
							        array(
						            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
							            array("nfeTools" => json_decode($nfeTools)),
							            array("NRACESSONFCE" => $nChave),
							            array("IDTPAMBNFCE" => $nota['IDTPAMBNFCE'])
							        )
							    );
							    
							    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
							    $respostaEnvio = json_decode($respostaEnvio);
							    if(!empty($respostaEnvio->dataset->return[0])){
							    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
							    	$result = $respostaEnvio['XMLRETORNO'];
							    }
							}
					     }
					}
			    }
	    	}
	    }
	    return $result;
	}
	
	public function recalculaChave($chaveAcesso, $permFEMISSAO = false){
		$NRACESSONFCE1 = substr($chaveAcesso, 0, 34);
		$FEMISSAO = substr($chaveAcesso, 34, 1);
        $NRACESSONFCE2 = substr($chaveAcesso, 35, 8);
        
        if(!$permFEMISSAO){
        	if($FEMISSAO == 9){
        		$FEMISSAO = 1;
	        }else{
	        	$FEMISSAO = 9;
	        }
        }
        
        $CHAVE = $NRACESSONFCE1.$FEMISSAO.$NRACESSONFCE2;
        $DIGITO = self::calculaDigitoVerificadorNFe($CHAVE); //$this->modulo11($NRACESSONFCE1.'9'.$NRACESSONFCE2);
        $modulo = $CHAVE.$DIGITO;
        
        return $modulo;
	}
	
	public function consultaReciboNFe($recibo, $nota){
		/*$nfeTools = $this->recuperarNfeTools($nota);
		$result = $nfeTools->sefazConsultaRecibo($recibo, $nota['IDTPAMBNFCE']);
		$stdCl = new Standardize($result);
		return $stdCl->toStd();*/
		
		$nfeTools = $this->recuperarNfeTools($nota);
		//ira consultar duas vezes, com a chave gravada no banco e com a chave altera
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("recibo" => $recibo),
	            array("IDTPAMBNFCE" => $nota['IDTPAMBNFCE'])
	        )
	    );
	    
	    
	    $rota = 'consultaRecidoNFCe';
	    $url  = $this->BASEURL.$rota;
	    $result = '';
	    
	    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
	    if(!empty($respostaEnvio)){
	    	$respostaEnvio = json_decode($respostaEnvio);
	    	if(!isset($respostaEnvio->error)){
			    if(!empty($respostaEnvio->dataset->return[0])){
			    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
			    	$result = $respostaEnvio['XMLRETORNO'];
			    	$stdCl = array();
			    	$stdCl['cStat'] = $this->getParameterXML($result, 'cStat');
			    	$stdCl['chNFe'] = $this->getParameterXML($result, 'chNFe');
			    	$stdCl['nRec'] = $this->getParameterXML($result, 'nRec');
			    	$stdCl['xMotivo'] = $this->getParameterXML($result, 'xMotivo');
					return (object) $stdCl;
			    }else{
			    	return null;
			    }
	    	}else{
	    		return null;
	    	}
	    }else{
	    	return null;
	    }
	}
	
	public function cancelarNota($nota) {
		$nfeTools = $this->recuperarNfeTools($nota);
		$chave = $nota['NRACESSONFCE'];
		$justf = $nota['JUSTIFICATIVA'];
		$prot = $nota['NRPROTOCOLONFCE'];

		if (strlen($justf) < 15) {
			throw new \Exception("Operação Bloqueada! A justificativa deve ter no mínimo 15 caracteres.");
		}
		
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("chave" => $chave),
	            array("protocolo" => $prot),
	            array("justificativa" => $justf)
	        )
	    );
	    
	    
	    $rota = 'cancelaNFCe';
	    $url  = $this->BASEURL.$rota;
	    $result = '';
	    
	    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
	    $respostaEnvio = json_decode($respostaEnvio);
	    if(!empty($respostaEnvio->dataset->return[0])){
	    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
	    	$result = $respostaEnvio['XMLRETORNO'];
	    }
		return ['xml' => $result];
	}
	
	public function cancelarNotaSubstituicao($nota) {
		
		$nfeTools = $this->recuperarNfeTools($nota);
		$chave = $nota['NRACESSONFCE'];
		$chNFeRef = $nota['NRACESSONFCE2'];
		$justf = "CANCELAMENTO POR SUBSTITUICAO";
		$prot = $nota['NRPROTOCOLONFCE'];
		$verAplic = 'RetailManagement';

		if (empty($justf)) {
			throw new \Exception("Operação Bloqueada! É necessário informar uma justificativa.");
		}
		
		if (empty($chNFeRef)) {
			throw new \Exception("Operação Bloqueada! É necessário informar a chave de substituição.");
		}
		
		$parameters = array("requestType" => "DataSet", "dataset" => 
	        array(
            array("CERTIFICADO" => base64_encode(file_get_contents($this->DSDIRCERTIFICADO)), "SENHA" => $this->CDSENHACERTIF),
	            array("nfeTools" => json_decode($nfeTools)),
	            array("chave" => $chave),
	            array("nProt" => $prot),
	            array("chNFeRef" => $chNFeRef),
	            array("verAplic" => $verAplic),
	            array("xJust" => $justf)
	        )
	    );
	    
	    $rota = 'cancelamentoSubstNFCe';
	    $url  = $this->BASEURL.$rota;
	    $result = '';
	    $respostaEnvio = $this->curlCall($url, json_encode($parameters));
	    if(!empty($respostaEnvio)){
	    	$respostaEnvio = json_decode($respostaEnvio);
	    	if(!isset($result->error)){
			    if(!empty($respostaEnvio->dataset->return[0])){
			    	$respostaEnvio = (array)$respostaEnvio->dataset->return[0];	
			    	if(!$respostaEnvio['error']){
			    		$result = $respostaEnvio['XMLRETORNO'];
				    	return ['xml' => $result, 'error' => false];		
			    	}else{
			    		return ['error' => true, 'message' => $respostaEnvio['message']];	
			    	}
			    }else{
			    	return ['error' => true, 'message' => 'Erro ao comunicar com serviço de cancelamento'];
			    }
	    	}
	    }
	}
	
	private function curlCall($url, $payload) {
		
	  try {
		
		    $ch = curl_init(); 
		    curl_setopt($ch, CURLOPT_URL, $url); 
		    curl_setopt($ch, CURLOPT_HEADER, TRUE); 
		    curl_setopt($ch, CURLOPT_NOBODY, TRUE); // remove body 
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE); 
		    curl_setopt($ch, CURLOPT_POST, 1); 
		    curl_setopt($ch, CURLOPT_POSTFIELDS, $payload); 
		    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		        'Content-Type: application/json',
		        'Content-Length: ' . strlen($payload))
		    );
		    $response = curl_exec($ch); 
		    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
		    
		    $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
		    $header = substr($response, 0, $header_size);
		    $body = substr($response, $header_size);
		    curl_close($ch);
		    
		    return $body;
		    
		}
		catch(Exception $e){
		    throw new Exception("Erro ao conectar com Serviço de Transmissão NFC-e Odhen",0,$e);
		}
	}
	
	private function noSpecials($texto){
		$valor = preg_replace('/\D/', '', $texto);
		return $valor;
	}
	
	private function getParameterXML($xml, $parametro) {
        
        $chave1 = '<'.$parametro.'>';
        $chave2 = '</'.$parametro.'>';
        
        if(strpos($xml, $chave1) && strpos($xml, $chave2)){
        	$posIn = strpos($xml, $chave1);
	        $posFi = strpos($xml, $chave2);
	        $dado = substr($xml, $posIn+strlen($chave1), $posFi-($posIn+strlen($chave1)));
	        return $dado;
        }else{
        	return null;
        }
        
    }
    
    private function valorMoeda($valorAplicado){
		
		$posVirgula = strpos( $valorAplicado, "," );
		$posPont = strpos( $valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		
		return $valorAplicado2;
	}
	
	private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
    public function formatValueNrlanctonfce($nrlanctonfce){
    	$value = $nrlanctonfce;
    	$sizeNrlanctonfce = strlen($nrlanctonfce);
    	$CD_LENGTH = 8;

    	if($sizeNrlanctonfce > 8){
    		$value = substr($nrlanctonfce, 2);
    	}elseif($sizeNrlanctonfce < 8){
            $value = self::leftZeroFill($value, $CD_LENGTH);
    	}
    	
    	return $value;
    }
	
	const GET_EMPRESAFILIAL = "
      SELECT F.CDINSCESTA, E.CDEMPRESA, E.CDPAIS, E.SGESTADO, E.CDMUNICIPIO, F.NRINSJURFILI, E.NMRAZSOCEMPR
        FROM FILIAL F, EMPRESA E
       WHERE F.CDFILIAL = :CDFILIAL
         AND F.NRORG    = :NRORG
         AND E.NRORG    = F.NRORG
         AND E.CDEMPRESA = F.CDEMPRESA
    ";
    
    const GET_ENDEFILI = "
        SELECT SGESTADO, CDMUNICIPIO, CDPAIS
           FROM ENDEFILI
          WHERE CDFILIAL     = :CDFILIAL
            AND IDTPENDEFILI = 'P'
            AND NRORG        = :NRORG
    ";
    
    const GET_DADOSEMITENTE_IDE_XML = "
        SELECT F.NRINSJURFILI, UPPER(F.NMRAZSOCFILI) NMRAZSOCFILI, UPPER(F.NMFILIAL) NMFILIAL, UPPER(EF.DSENDEFILI) DSENDEFILI,
               EF.NRENDEFILISEP,UPPER(EF.NMBAIRFILI) NMBAIRFILI,M.CDMUNICIBGE,UPPER(M.NMMUNICIPIO) NMMUNICIPIO, EF.SGESTADO,
               EF.NRCEPFILI, P.CDPAISBACEN,UPPER(P.NMPAIS) NMPAIS,F.CDINSCESTA,F.CDINSCMUNI,E.CDCNAE,PE.CDSITUCRT, ES.CDESTADOIBGE,
               PV.CDPRODTAXSER, PV.IDAMBTRABNFCE, PV.NRCERTDIGNFCE, PV.CDIDTOKENPROD, PV.NMARQCERTNFCE, PV.DSSENHACERTNFCE, PV.CDCODSCONSPROD,
               PV.CDCODSCONSHOMO, PV.CDIDTOKENHOMO, PV.CDURLWSNFC, PV.IDRESPTECNFCE
          FROM FILIAL F, ENDEFILI EF, MUNICIPIO M, PAIS P, PARAMEMPRESA PE, PARAVEND PV,
               EMPRESA E, ESTADO ES
         WHERE F.CDFILIAL      = PV.CDFILIAL
           AND F.CDFILIAL      = :CDFILIAL
           AND F.NRORG         = :NRORG
           AND EF.CDFILIAL     = F.CDFILIAL
           AND EF.NRORG        = F.NRORG
           AND EF.IDTPENDEFILI = 'P'
           AND ES.SGESTADO     = EF.SGESTADO
           AND ES.CDPAIS       = EF.CDPAIS
           AND M.CDMUNICIPIO   = EF.CDMUNICIPIO
           AND M.SGESTADO      = EF.SGESTADO
           AND P.CDPAIS        = EF.CDPAIS
           AND E.CDEMPRESA     = F.CDEMPRESA
           AND E.NRORG         = F.NRORG
           AND PE.CDEMPRESA    = E.CDEMPRESA
           AND PE.NRORG        = E.NRORG
    ";
    
    const GET_SERIE_NFCE = "
        SELECT CDSERIECX, CDCAIXA
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA  = :CDCAIXA
           AND NRORG    = :NRORG
    ";
    
    const GET_PRODUCTS_TO_PRINT_NFCE = "
          SELECT IV.NRSEQUITVEND, IV.QTPRODVEND, IV.VRUNITVEND, IV.VRUNITVENDCL,
                 IM.IDTPIMPOSFIS, IP.VRPEALPRODIT,  NVL(IV.VRDESITVEND,0) AS VRTOTDESC,
                 PR.NMPRODUTO, PR.SGUNIDADE, LJ.CDPRODTAXASERV,
                 (IV.VRUNITVEND+IV.VRUNITVENDCL + NVL((case
                                             when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
                             						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then IV.VRACRITVEND
                                             else 0
                                              end), 0)) AS VRTOTITEM,
                  NVL((case
					   when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then 0
				   else
					   IV.VRACRITVEND
				   end), 0) AS VRTOTACRE,
                 CASE WHEN VE.VRTXSEVENDA IS NULL THEN 0 ELSE VE.VRTXSEVENDA END AS VRTXSEVENDA,
                 PR.CDPRODUTO, CASE WHEN PR.CDBARPRODUTO IS NULL THEN PR.CDARVPROD ELSE PR.CDBARPRODUTO END AS CDARVPROD,
                 SUM(CASE MC.IDTIPOMOVIVE
                     WHEN 'E' THEN MC.VRMOVIVEND
                     WHEN 'T' THEN MC.VRMOVIVEND
                     ELSE 0
                      END) AS VRENTRADA,
                 SUM(CASE MC.IDTIPOMOVIVE
                     WHEN 'S' THEN MC.VRMOVIVEND
                     ELSE 0
                      END) AS VRSAIDA
            FROM VENDA VE
            JOIN ITEMVENDA IV ON IV.CDFILIAL   = VE.CDFILIAL
                             AND IV.CDCAIXA    = VE.CDCAIXA
                             AND IV.NRSEQVENDA = VE.NRSEQVENDA

            JOIN PRODUTO PR ON PR.CDPRODUTO  = IV.CDPRODUTO
            JOIN MOVCAIXA MC ON MC.CDFILIAL   = VE.CDFILIAL
                            AND MC.CDCAIXA    = VE.CDCAIXA
                            AND MC.NRSEQVENDA = VE.NRSEQVENDA
            JOIN ITVENDAIMPOS IP ON IP.CDFILIAL   = VE.CDFILIAL
                                AND IP.CDCAIXA    = VE.CDCAIXA
                                AND IP.NRSEQVENDA = VE.NRSEQVENDA
                                AND IP.NRSEQUITVEND = IV.NRSEQUITVEND
            JOIN IMPOSTO IM ON IM.CDIMPOSTO  = IP.CDIMPOSTO
            JOIN LOJA LJ ON LJ.CDFILIAL = VE.CDFILIAL
                AND LJ.CDLOJA = VE.CDLOJA
          WHERE
            VE.CDFILIAL   = :CDFILIAL
            AND VE.CDCAIXA    = :CDCAIXA
                    AND VE.NRSEQVENDA = :NRSEQVENDA
          GROUP BY
            IV.NRSEQUITVEND, IV.QTPRODVEND, IV.VRUNITVEND, IV.VRUNITVENDCL, IV.VRACRITVEND, IV.VRRATTXSERV, IV.VRDESITVEND, VE.VRTXSEVENDA,
            PR.CDPRODUTO, PR.CDBARPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, IM.IDTPIMPOSFIS, IP.VRPEALPRODIT, LJ.CDPRODTAXASERV, 
            (IV.VRUNITVEND+IV.VRUNITVENDCL + NVL((case
                                             when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
                             						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then IV.VRACRITVEND
                                             else 0
                                              end), 0)), 
             NVL((case
					   when (LJ.CDPRODTAXASERV = PR.CDPRODUTO) or
						     (LJ.CDPRODTAXAENTR = PR.CDPRODUTO) then 0
				   else
					   IV.VRACRITVEND
				   end), 0)
          ORDER BY
            IV.NRSEQUITVEND
      ";
      
    const GET_CONSUMER_SALE = "
          SELECT
            VE.CDFILIAL, VE.CDCAIXA, VE.NRSEQVENDA, VE.DTENTRVENDA, VE.NRNOTAFISCALCE,
            VE.NRACESSONFCE, VE.NRINSCRCONS, VE.NMCONSVEND, VE.CDCLIENTE, VE.CDCONSUMIDOR,
            CO.NMCONSUMIDOR, CO.NRCPFRESPCON, CL.IDTPIJURCLIE, CL.NRINSJURCLIE, CO.VRMAXDEBCONS,
            CO.VRMAXCREDCONS, CO.VRMESUBCONS, CL.NMFANTCLIE, VE.DSENDECONSVENDA, IE.NMBAIRIEST,
            MN.CDMUNICIBGE, MN.NMMUNICIPIO, ES.CDESTADOIBGE, ES.SGESTADO
          FROM
            VENDA VE JOIN FILIAL FI
                      ON VE.CDFILIAL = FI.CDFILIAL
            LEFT JOIN INSCRESTAD IE
                      ON FI.CDINSCESTA = IE.CDINSCESTA
                    AND FI.CDEMPRESA = IE.CDEMPRESA
            LEFT JOIN ESTADO ES
                    ON IE.CDPAIS = ES.CDPAIS
                    AND IE.SGESTADO = ES.SGESTADO
            LEFT JOIN MUNICIPIO MN
                      ON IE.CDPAIS = MN.CDPAIS
                      AND IE.SGESTADO = MN.SGESTADO
                      AND IE.CDMUNICIPIO = MN.CDMUNICIPIO
            LEFT JOIN CLIENTE CL
                      ON VE.CDCLIENTE = CL.CDCLIENTE
            LEFT JOIN CONSUMIDOR CO
                      ON VE.CDCLIENTE = CO.CDCLIENTE
                    AND VE.CDCONSUMIDOR = CO.CDCONSUMIDOR
          WHERE
            (VE.CDFILIAL   = :CDFILIAL  ) AND
            (VE.CDCAIXA    = :CDCAIXA   ) AND
            (VE.NRSEQVENDA = :NRSEQVENDA)";
            
    // const GET_DADOSPRODIMP_XML = "
    //     SELECT I.CDFILIAL,
    //         I.CDIMPOSTO, AL.CDCFOPPFIS, P.CDCEST, NVL(AL.VRPERCREDUCAO, 0) VRPERCREDUCAO,
    //         I.CDCAIXA, I.NRSEQVENDA, I.NRSEQUITVEND, I.VRPEALPRODIT, I.VRPERPIS, AL.VRALIQPIS,
    //         I.VRPERCOFINS, I.CDCSTPRODI, I.CDCSTPRODPC, I.VRBASECALCICMS, P.CDARVPROD,
    //         P.NMPRODUTO, P.CDCLASFISC, P.SGUNIDADE, I.VRTOTTRIBIBPT
    //       FROM ITVENDAIMPOS I
    //         JOIN PRODUTO P
    //         ON P.CDPRODUTO = :CDPRODUTO
    //         AND P.NRORG    = I.NRORG
    //       JOIN ALIQIMPFIS AL
    //         ON AL.CDPRODUTO    = P.CDPRODUTO
    //         AND AL.CDFILIAL    = I.CDFILIAL
    //         AND AL.NRORG       = I.NRORG
    //       WHERE I.CDFILIAL   = :CDFILIAL
    //         AND I.CDCAIXA      = :CDCAIXA
    //         AND I.NRSEQVENDA   = :NRSEQVENDA
    //         AND I.NRSEQUITVEND = :NRSEQUITVEND
    //         AND I.NRORG        = :NRORG";
            
    const GET_DADOSPRODIMP_XML = "
        SELECT I.CDFILIAL,
            I.CDIMPOSTO, AL.CDCFOPPFIS, P.CDCEST, NVL(AL.VRPERCREDUCAO, 0) VRPERCREDUCAO,
            I.CDCAIXA, I.NRSEQVENDA, I.NRSEQUITVEND, I.VRPEALPRODIT, I.VRPERPIS, AL.VRALIQPIS,
            I.VRPERCOFINS, I.CDCSTPRODI, I.CDCSTPRODPC, I.VRBASECALCICMS, P.CDARVPROD,
            P.NMPRODUTO, P.CDCLASFISC, P.SGUNIDADE, I.VRTOTTRIBIBPT, AL.VRPEALIMPFIS, AL.IDTPIMPOSFIS, AL.VRALIQIBPT, AL.VRALIQIBPTES, AL.CDCFOPPFIS, AL.CDCSTICMS, AL.CDCSTPISCOF,
            AL.VRALIQPIS, AL.VRALIQCOFINS, AL.VRPERCREDUCAO, NVL(AL.CDCEST,P.CDCEST), AL.VRPEALIMPFISEF, AL.VRPERCREDUCAOEF,
            I.VRTOTTRIBIBPT, AL.CDCBENEF, I.VRBASECALCICMS, I.VRBCPISCOFINS, I.VRBASECALCREDUZ,
            I.VRIMPPIS, I.VRIMPCOFINS, I.VRIMPICMSFCP, AL.VRALIQFCP,
            I.VRPEALEFPRODIT, I.VRIMPOEFPRODIT, I.VRBCREDUZEFICMS, I.VRPRBCREDUEFICMS,
            AL.VRALIQICMSDES
          FROM ITVENDAIMPOS I
            JOIN PRODUTO P
            ON P.CDPRODUTO = :CDPRODUTO
            AND P.NRORG    = I.NRORG
          JOIN ALIQIMPFIS AL
            ON AL.CDPRODUTO    = P.CDPRODUTO
            AND AL.CDFILIAL    = I.CDFILIAL
            AND AL.NRORG       = I.NRORG
          WHERE I.CDFILIAL   = :CDFILIAL
            AND I.CDCAIXA      = :CDCAIXA
            AND I.NRSEQVENDA   = :NRSEQVENDA
            AND I.NRSEQUITVEND = :NRSEQUITVEND
            AND I.NRORG        = :NRORG";
            
    const GET_PAYMENTS = "
		SELECT TR.NMTIPORECE, MC.VRMOVIVEND, TR.IDTIPORECE, MC.IDTIPOMOVIVE, TR.CDBANCARTCR, TR.CDADMINCART,
            MC.CDNSUHOSTTEF, MC.QTPARCRECEB, TR.CDBANCARTSEFAZ, TR.IDDESABTEF
          FROM TIPORECE TR, MOVCAIXA MC
        WHERE MC.CDFILIAL     = :CDFILIAL
            AND MC.CDCAIXA    = :CDCAIXA
            AND MC.NRSEQVENDA = :NRSEQVENDA
            AND TR.CDTIPORECE = MC.CDTIPORECE";
            
    const UPDATE_VENDA_NFCE = "
        UPDATE VENDA
          SET CDSERIENFCE = :CDSERIENFCE,
            IDTPAMBNFCE = :IDTPAMBNFCE,
            NRACESSONFCE = :NRACESSONFCE,
            NRNOTAFISCALCE = :NRNOTAFISCALCE,
            NRLANCTONFCE = :NRLANCTONFCE,
            IDSTATUSNFCE = :IDSTATUSNFCE,
            NRPROTOCOLONFCE = :NRPROTOCOLONFCE,
            DSOBSSTATUSNFCE = :DSOBSSTATUSNFCE,
            IDMODOPERACNFCE = :IDMODOPERACNFCE,
            NRRECIBONFCE = :NRRECIBONFCE,
            CDOPERENVIONFCE = :CDOPERENVIONFCE,
            DTHRPROTOCONFCE = :DTHRPROTOCONFCE,
            IDXMLVALIDNFCE = :IDXMLVALIDNFCE,
            DSQRCODENFCE = :DSQRCODENFCE,
            DSARQXMLNFCE = :DSARQXMLNFCE
        WHERE NRORG = :NRORG
            AND NRSEQVENDA = :NRSEQVENDA
            AND CDFILIAL = :CDFILIAL
            AND CDCAIXA = :CDCAIXA";
            
    const GET_LINK_CERTIFICADO = "
        SELECT P.FILESERVERURL || 
		       SUBSTR(I.DSDIRCERTIFICADO, INSTR(I.DSDIRCERTIFICADO, '>')+1) DSDIRCERTIFICADO, I.CDSENHACERTIF, I.DSDIRCERTIFICADO CERTIFICADO, 
		       (SELECT DSSENHACERTNFCE FROM PARAVEND WHERE CDFILIAL = :CDFILIAL) AS DSSENHACERTNFCE
		  FROM INSCRESTAD I, PARAMGERAL P
		 WHERE I.CDINSCESTA = :CDINSCESTA";
		 
	const UPDATE_VENDA_STATUS_NFCE_CAN = "
	   UPDATE VENDA
          SET DSOBSSTATUSNFCE = :DSOBSSTATUSNFCE, NRPROTOCOLOCANC = :NRPROTOCOLOCANC, IDSTATUSNFCE = 'C',
            DTHRPROTOCOCANC = :DTHRPROTOCOCANC, DSRAZAOCANCNFCE = :DSRAZAOCANCNFCE,
            DSARQXMLCANCNFCE = :DSARQXMLCANCNFCE, CDOPERADORCANC = :CDOPERADORCANC
        WHERE CDFILIAL = :CDFILIAL
          AND CDCAIXA = :CDCAIXA
          AND NRSEQVENDA = :NRSEQVENDA";
          
	const UPDATE_VENDA_STATUS_NFCE_CAN_SB = "
	   UPDATE VENDA
          SET DSOBSSTATUSNFCE = :DSOBSSTATUSNFCE, NRPROTOCOLOCANC = :NRPROTOCOLOCANC, IDSTATUSNFCE = :IDSTATUSNFCE,
            DTHRPROTOCOCANC = :DTHRPROTOCOCANC, DSRAZAOCANCNFCE = :DSRAZAOCANCNFCE,
            DSARQXMLCANCNFCE = :DSARQXMLCANCNFCE, CDOPERADORCANC = :CDOPERADORCANC
        WHERE CDFILIAL = :CDFILIAL
          AND CDCAIXA = :CDCAIXA
          AND NRSEQVENDA = :NRSEQVENDA";

}